//=============================================================================
// mi_DifficultySelection.js
//=============================================================================


/*:
 * @plugindesc
 * @author micoomer
 *
 */


function Scene_Difficulty() {
    this.initialize.apply(this, arguments);
}

Scene_Difficulty.prototype = Object.create(Scene_MenuBase.prototype);
Scene_Difficulty.prototype.constructor = Scene_Difficulty;

Scene_Difficulty.prototype.initialize = function() {
    Scene_MenuBase.prototype.initialize.call(this);
    this._phase = 0;
    this._sel_org = [605, 1046];
    this._con_org = [421, 862];
    this._con_time = 10;
};

Scene_Difficulty.prototype.create_mbackground = function() {
};

Scene_Difficulty.prototype.create = function() {
  Scene_MenuBase.prototype.create.call(this);
  this.loadBitmaps();

  this.createWindow_Panel();
  this.createWindow_Confirm();

  this.createField();
  this.createTitles();
  this.createPanel();
};

Scene_Difficulty.prototype.loadBitmaps = function() {
  this._spr = ImageManager.loadBackground("UI_Difficulty");
};

Scene_Difficulty.prototype.createWindow_Panel = function() {
    var wx = 230;
    var wy = 125;
    this._w_panel = new Window_DiffPanel(wx, wy);
    this._w_panel.setHandler('safe',     this.onSafe.bind(this));
    this._w_panel.setHandler('risky',    this.onRisky.bind(this));
    this._w_panel.deactivate();
    this._w_panel.hide();
    this._w_panel.contentsOpacity = 0;
    this._w_panel.opacity = 0;
    this.addWindow(this._w_panel);
};

Scene_Difficulty.prototype.createWindow_Confirm = function() {
    var wx = 230;
    var wy = 125;
    this._w_confirm = new Window_DiffConfirm(wx, wy);
    this._w_confirm.setHandler('yes',     this.onConfirmOk.bind(this));
    this._w_confirm.setHandler('cancel',  this.onConfirmCancel.bind(this));
    this._w_confirm.deactivate();
    this._w_confirm.hide();
    this._w_confirm.contentsOpacity = 0;
    this._w_confirm.opacity = 0;
    this.addWindow(this._w_confirm);
};

Scene_Difficulty.prototype.onSafe = function() {
  this._w_confirm.x = 230;
  this._w_panel.deactivate();
  this._w_panel.hide();
  this._phase = 3;
};

Scene_Difficulty.prototype.onRisky = function() {
  this._w_confirm.x = 670;
  this._w_panel.deactivate();
  this._w_panel.hide();
  this._phase = 3;
};

Scene_Difficulty.prototype.onConfirmOk = function() {
  $gameSystem._difficulty = Micoomer._difficultyArray[this._w_panel._index];
  this._phase = 5;
};

Scene_Difficulty.prototype.onConfirmCancel = function() {
  this._w_confirm.deactivate();
  this._w_confirm.hide();
  this._w_panel.activate();
  this._w_panel.show();
  this._ok.scale.x = 0;
  this._con_time = 10;
};

Scene_Difficulty.prototype.activatePanelWindow = function() {
  this._w_panel.activate();
  this._w_panel.show();
  this._w_panel.select(0);
};

Scene_Difficulty.prototype.activateConfirmationWindow = function() {
  this._w_confirm.activate();
  this._w_confirm.show();
};

Scene_Difficulty.prototype.createField = function() {
	this._f = new Sprite();
	this.addChild(this._f);
};

Scene_Difficulty.prototype.createTitles = function() {
  this._bg = [];
	this._bg = new Sprite(this._spr);
  this._bg.opacity = 0;
  this._bg.anchor.x = 0.5;
  this._bg.x = 640;
  this._bg.y = 64;
  this._bg.setFrame(0,0,590,71);
	this._f.addChild(this._bg);
};

Scene_Difficulty.prototype.createPanel = function() {
	this._panel = new Sprite(this._spr);
  this._panel.opacity = 0;
  this._panel.anchor.x = 0.5;
  this._panel.x = 640;
  this._panel.y = 169;
  this._panel.setFrame(0,71,817,518);
  this._f.addChild(this._panel);
};

Scene_Difficulty.prototype.createSelection = function() {
	this._selection = new Sprite(this._spr);
  this._selection.setFrame(0,589,512,370);
  this._selection.anchor.x = 0.5;
  this._selection.x = 1046;
  this._selection.y = 402;
  this._selection.rotation = (Math.PI / 180) * 90;
  this._f.addChild(this._selection);
};

Scene_Difficulty.prototype.createOk = function() {
	this._ok = new Sprite(this._spr);
  this._ok.setFrame(512,595,71,361);
  this._ok.anchor.y = 0.5;
  this._ok.anchor.x = 1.0;
  this._ok.scale.x = 0;
  this._ok.y = 656;
  this._ok.rotation = (Math.PI / 180) * 90;
  this._f.addChild(this._ok);
};

Scene_Difficulty.prototype.updatePhase = function() {
  switch (this._phase) {
    case 0:
      this._bg.opacity += 10;
      this._bg.y -= 1;
      this._panel.opacity += 10;
      this._panel.y -= 1;
      if (this._bg.opacity == 255) {this._phase = 1; this.createSelection()};
      break;

    case 1:
      if (this._selection) {
        this._selection.opacity += 10;
        if (this._selection.opacity == 255) {
          this._phase = 2;
          this.activatePanelWindow();
          this.createOk();
        };
      }
      break;

    case 3:
      if (this._ok) {
        this._ok.scale.x += 0.1;
        this._con_time--;
        if (this._con_time == 0) {this._phase = 4; this.activateConfirmationWindow()};
      }
      break;

    case 5:
      this._f.opacity -= 10;
      if (this._f.opacity <= 0) SceneManager.pop();
      break;
  }
}

Scene_Difficulty.prototype.updateSel = function() {
  if (this._selection) this._selection.x = this._sel_org[this._w_panel._index];
  if (this._ok) this._ok.x = this._con_org[this._w_panel._index];
};

Scene_Difficulty.prototype.update = function() {
  Scene_MenuBase.prototype.update.call(this);
  this.updatePhase();
  this.updateSel();
};


//=============================================================================
// * Window
//=============================================================================

function Window_DiffPanel() {
    this.initialize.apply(this, arguments);
}

Window_DiffPanel.prototype = Object.create(Window_Command.prototype);
Window_DiffPanel.prototype.constructor = Window_DiffPanel;

Window_DiffPanel.prototype.initialize = function(x, y) {
  Window_Command.prototype.initialize.call(this, x, y);
};

Window_DiffPanel.prototype.maxCols = function() {
    return 2;
};

Window_DiffPanel.prototype.windowWidth = function() {
    return 820;
};

Window_DiffPanel.prototype.lineHeight = function() {
    return 520;
};

Window_DiffPanel.prototype.spacing = function() {
    return 80;
};

Window_DiffPanel.prototype.makeCommandList = function() {
  this.addCommand('safe', 'safe', true);
  this.addCommand('risky', 'risky', true);
};

function Window_DiffConfirm() {
    this.initialize.apply(this, arguments);
}

Window_DiffConfirm.prototype = Object.create(Window_Command.prototype);
Window_DiffConfirm.prototype.constructor = Window_DiffConfirm;

Window_DiffConfirm.prototype.initialize = function(x, y, width, height) {
  Window_Command.prototype.initialize.call(this, x, y, width, height);
};

Window_DiffConfirm.prototype.maxCols = function() {
    return 1;
};

Window_DiffConfirm.prototype.windowWidth = function() {
    return 380;
};

Window_DiffConfirm.prototype.lineHeight = function() {
    return 520;
};

Window_DiffConfirm.prototype.makeCommandList = function() {
  this.addCommand('yes', 'yes', true);
};
