Bitmap.prototype.initialize = function(width, height) {
    if(!this._defer){
        this._createCanvas(width, height);
    }

    this._image = null;
    this._url = '';
    this._paintOpacity = 255;
    this._smooth = false;
    this._loadListeners = [];
    this._loadingState = 'none';
    this._decodeAfterRequest = false;

    /**
     * Cache entry, for images. In all cases _url is the same as cacheEntry.key
     * @type CacheEntry
     */
    this.cacheEntry = null;

    /**
     * The face name of the font.
     *
     * @property fontFace
     * @type String
     */
    this.fontFace = 'GameFont';

    /**
     * The size of the font in pixels.
     *
     * @property fontSize
     * @type Number
     */
    this.fontSize = 28;

    /**
     * Whether the font is italic.
     *
     * @property fontItalic
     * @type Boolean
     */
    this.fontItalic = false;

    /**
     * The color of the text in CSS format.
     *
     * @property textColor
     * @type String
     */
    this.textColor = '#ffffff';

    /**
     * The color of the outline of the text in CSS format.
     *
     * @property outlineColor
     * @type String
     */
    this.outlineColor = 'rgba(0, 0, 0, 0)';

    /**
     * The width of the outline of the text.
     *
     * @property outlineWidth
     * @type Number
     */
    this.outlineWidth = 0;
};

ImageManager.loadBitmap = function(folder, filename, hue, smooth) {
    if (filename) {
        var path = folder + encodeURIComponent(filename) + '.png';
        var bitmap = this.loadNormalBitmap(path, hue || 0);
        //console.log("disabling smoothing for "+path);
        bitmap.smooth = false; return bitmap;
    } else {
        return this.loadEmptyBitmap();
    }
};

//-----------------------------------------------------------------------------
// Window_Base
//-----------------------------------------------------------------------------

Window_Base.prototype.paramchangeTextColor = function(change) {
    if (change > 0) {
        return this.powerUpColor();
    } else if (change < 0) {
        return this.powerDownColor();
    } else {
        return this.textColor(1);
    }
};

Window_Base.prototype.drawTextEx = function(text, x, y) {
    if (text) {
        var textState = { index: 0, x: x, y: y, left: x };
        textState.text = this.convertEscapeCharacters(text);
        textState.height = 20;
        this.resetFontSettings();
        while (textState.index < textState.text.length) {
            this.processCharacter(textState);
        }
        return textState.x - x;
    } else {
        return 0;
    }
};

//-----------------------------------------------------------------------------
// Scene_Options
//-----------------------------------------------------------------------------

ImageManager.loadMenusOption = function(filename) {
    return this.loadBitmap('img/menus/option/', filename, 0, true);
};

Scene_Options.prototype.create = function() {
    Scene_MenuBase.prototype.create.call(this);
	this.createLayout();
    this.createOptionsWindow();
};

Scene_Options.prototype.createLayout = function() {
    this._optionlayout = new Sprite(ImageManager.loadMenusOption("Layout"));
	this.addChild(this._optionlayout);
};

Scene_Options.prototype.createOptionsWindow = function() {
    this._optionsWindow = new Window_Options();
	this._optionsWindow.x += 8;
	this._optionsWindow.opacity = 0;
    this._optionsWindow.setHandler('cancel', this.popScene.bind(this));
    this.addChild(this._optionsWindow);
};

Window_Options.prototype.makeCommandList = function() {
  this.addVolumeOptions();
  this.addCommand(TextManager.alwaysDash, 'alwaysDash');
};

Window_Options.prototype.drawItem = function(index) {
    var rect = this.itemRectForText(index);
    var statusWidth = this.statusWidth();
    var titleWidth = rect.width - statusWidth;
    this.resetTextColor();
	this.changeTextColor(this.textColor(1));
	this.contents.fontSize = 24;
    this.changePaintOpacity(this.isCommandEnabled(index));
    this.drawText(this.commandName(index), rect.x, rect.y, titleWidth, 'left');
    this.drawText(this.statusText(index), titleWidth, rect.y, statusWidth, 'right');
};

Window_Options.prototype.isCursorVisible = function() {
    return false;
};

//-----------------------------------------------------------------------------
// Window_SkillType
//-----------------------------------------------------------------------------

Window_SkillType.prototype.lineHeight = function(index) {
	return 72;
};

Window_SkillType.prototype.isCursorVisible = function() {
    return false;
};

//-----------------------------------------------------------------------------
// Scene_Equip
//-----------------------------------------------------------------------------

Scene_Equip.prototype.create = function() {
    Scene_MenuBase.prototype.create.call(this);
    this.createHelpWindow();
    this.createStatusWindow();
    this.createSlotWindow();
    this.createItemWindow();
    this.refreshActor();
	  this.commandEquip();
};

Scene_Equip.prototype.createSlotWindow = function() {
    var wx = this._statusWindow.width;
    var wy = this._helpWindow.height;
    var ww = Graphics.boxWidth - this._statusWindow.width;
    var wh = this._statusWindow.height - 64;
    this._slotWindow = new Window_EquipSlot(wx, wy, ww, wh);
    this._slotWindow.setHelpWindow(this._helpWindow);
    this._slotWindow.setStatusWindow(this._statusWindow);
    this._slotWindow.setHandler('ok',       this.onSlotOk.bind(this));
    this._slotWindow.setHandler('cancel',   this.onSlotCancel.bind(this));
	  this._slotWindow.setHandler('pagedown', this.nextActor.bind(this));
    this._slotWindow.setHandler('pageup',   this.previousActor.bind(this));
    this.addWindow(this._slotWindow);
};

Scene_Equip.prototype.onSlotCancel = function() {
    SceneManager.pop()
};

Scene_Equip.prototype.onActorChange = function() {
    this.refreshActor();
    this.commandEquip();
	this._slotWindow.refresh();
	this._itemWindow.deselect();
	this._itemWindow.refresh();
	this._statusWindow.refresh();
};

Window_EquipStatus.prototype.drawCurrentParam = function(x, y, paramId) {
	this.changeTextColor(this.textColor(1));
	this.contents.fontFace = 'FredokaSemi';
    if (paramId == 5) {
		this.drawText(this._actor.param(paramId) + "%", x, y, 70, 'right');
	} else {
		this.drawText(this._actor.param(paramId), x, y, 70, 'right');
	}
	this.resetTextColor();
};

Window_EquipStatus.prototype.drawNewParam = function(x, y, paramId) {
    var newValue = this._tempActor.param(paramId);
    var diffvalue = newValue - this._actor.param(paramId);
    this.changeTextColor(this.paramchangeTextColor(diffvalue));
	if (paramId == 5) {
		this.drawText(newValue + "%", x, y, 70, 'right');
	} else {
		this.drawText(newValue, x, y, 70, 'right');
	}
};

Window_EquipSlot.prototype.isCursorVisible = function() {
    return false;
};

//-----------------------------------------------------------------------------
// Scene_File
//-----------------------------------------------------------------------------

ImageManager.loadMenusFile = function(filename) {
    return this.loadBitmap('img/menus/file/', filename, 0, true);
};

Scene_File.prototype.create = function() {
    Scene_MenuBase.prototype.create.call(this);
    DataManager.loadAllSavefileImages();
	this.createLayout();
    this.createListWindow();
};

Scene_File.prototype.createLayout = function() {
    this._filelayout = new Sprite(ImageManager.loadMenusFile("Layout"));
	this.addChild(this._filelayout);
};

Scene_File.prototype.createListWindow = function() {
    var x = 332;
    var y = 63;
    var width = 616;
    var height = 590;
    this._listWindow = new Window_SavefileList(x, y, width, height);
    this._listWindow.setHandler('ok',     this.onSavefileOk.bind(this));
    this._listWindow.setHandler('cancel', this.popScene.bind(this));
    this._listWindow.select(this.firstSavefileIndex());
    this._listWindow.setTopRow(this.firstSavefileIndex() - 2);
    this._listWindow.setMode(this.mode());
    this._listWindow.refresh();
    this.addChild(this._listWindow);
};

Window_SavefileList.prototype.isCursorVisible = function() {
    return false;
};

//------------------------------------------------------------------------------
//Sprite_Actor.prototype.updateSelectionEffect = function() {};
//Sprite_Enemy.prototype.updateSelectionEffect = function() {};


Window_Message.prototype.normalColor = function() {
    return this.textColor(0);
};

Window_ChoiceList.prototype.normalColor = function() {
    return this.textColor(0);
};

Window_DebugRange.prototype.normalColor = function() {
    return this.textColor(0);
};

Window_DebugEdit.prototype.normalColor = function() {
    return this.textColor(0);
};


// PLAYER MOVESPEED

Game_CharacterBase.prototype.realMoveSpeed = function() {
    return this._moveSpeed;
};

Game_CharacterBase.prototype.distancePerFrame = function() {
	if (this.isDashing()) {
		return Math.pow(2,this.realMoveSpeed()) / 128;
	}
	return Math.pow(2,this.realMoveSpeed()) / 256;
};


// FUMBLE SYSTEM
Sprite_Damage.prototype.setup = function(target) {
    var result = target.result();
    if (result.hpAffected) {
        if (result.fumble) {
          this.createDigits(1, result.hpDamage);
        } else {
          this.createDigits(0, result.hpDamage);
          if (result.critical) {
              this.setupCriticalEffect();
          }
        }
    } else if (target.isAlive() && result.mpDamage !== 0) {
        this.createDigits(2, result.mpDamage);
    }
};

Game_Action.prototype.itemFumble = function(target) {
    return Math.max(this.subject().hit - target.eva,0);
};

Game_Action.prototype.apply = function(target) {
    var result = target.result();
    this.subject().clearResult();
    result.clear();
    result.physical = this.isPhysical();
    result.drain = this.isDrain();
    result.fumble = (Math.random() > this.itemFumble(target));
    if (this.item().damage.type > 0) {
      result.critical = (Math.random() < this.itemCri(target));
      var value = this.makeDamageValue(target, result.critical);
      this.executeDamage(target, value);
    }
    this.item().effects.forEach(function(effect) {
      this.applyItemEffect(target, effect);
    }, this);
    this.applyItemUserEffect(target);
};

Game_ActionResult.prototype.clear = function() {
    this.used = false;
    this.missed = false;
    this.evaded = false;
    this.physical = false;
    this.drain = false;
    this.critical = false;
    this.success = false;
    this.hpAffected = false;
    this.fumble = false;
    this.hpDamage = 0;
    this.mpDamage = 0;
    this.tpDamage = 0;
    this.addedStates = [];
    this.removedStates = [];
    this.addedBuffs = [];
    this.addedDebuffs = [];
    this.removedBuffs = [];
};

// DIALOG CONDITION CHECKING
Game_Interpreter.prototype.dialog_conditionChecking = function(poplv) {
  var label = '';
  var unit = $gameParty.leader();
  var unitId = unit._actorId;
  switch (unitId) {
    case 1: label = 'Marie';break;
    case 2: label = 'Elucia';break;
    case 3: label = 'Ran';break;
  }

  var bellysize = unit.bellysize;
  var b
  if (bellysize <= 2) {b = 1}
  if (bellysize >= 3 && bellysize <= 4) {b = 2}
  if (bellysize >= 5) {b = 3}
  label += '_B' + b;

  var slutlevel = unit.lewdlevel;
  var s
  if (slutlevel <= 15) {s = 1}
  if (slutlevel >= 16 && slutlevel <= 30) {s = 2}
  if (slutlevel >= 31 && slutlevel <= 50) {s = 3}
  if (slutlevel >= 51) {s = 4}
  label += '_S' + s;

  var popularity = $gameSystem._popLv;
  var t
  switch (poplv) {
    case 0: t = 'Town';break;
    case 1: t = 'Brothel';break;
    case 2: t = 'Slum';break;
  }
  var p
  var pp = popularity[poplv];
  if (pp <= 1) {p = 1}
  if (pp >= 2 && pp <= 5) {p = 2}
  if (pp >= 6 && pp <= 8) {p = 3}
  if (pp >= 9) {p = 4}
  label += '_' + t + p;

  this._params = [String(label)];
  this.command119();
};

// ========================================================================================
// Game_Actor
// ========================================================================================
Micoomer.GameActorSetup = Game_Actor.prototype.setup;
Game_Actor.prototype.setup = function(actorId) {
    Micoomer.GameActorSetup.call(this, actorId);
    this._isBlessingActive = false;
    this._isPregnant = false;
    this._isDarkNipple = false;
    this._isOverdue = false;

    this._firstTimeSex = true;
    this._firstTimeBirth = true;

    this._prevEJP = 0;
    this._canSkill = true;
    this._canItem = true;
};

Game_Actor.prototype.canSkill = function() {
  return this._canSkill;
};

Game_Actor.prototype.canItem = function() {
  return this._canItem;
};

Game_Actor.prototype.canPass = function() {
  var skill = $dataSkills[2];
  return this.meetsSkillConditionsEval(skill);
};

Game_Actor.prototype.isBlessingActive = function() {
    return this._isBlessingActive;
};

Game_Actor.prototype.isPregnant = function() {
    return this._isPregnant;
};

Game_Actor.prototype.isDarkNipple = function() {
    return this._isDarkNipple;
};

Game_Actor.prototype.isOverdue = function() {
    return this._isOverdue;
};

Game_Actor.prototype.recoverStamina = function(amount) {
  var gain = this.stam + amount;
  if (gain > this.maxstam) {
    this.setCParam(8, this.maxstam);
  } else {
    this.addCParam(8, amount);
  }
};

Game_Actor.prototype.gainLewdness = function(amount) {
	var lw_gain = amount;
	while (lw_gain > 0) {
		var lw_lv = this.lewdlevel;
		var nextXP = Micoomer.lewdnessRequiredExp[lw_lv];
		var gain = this.lewdexp + lw_gain;
		var x = gain - nextXP;
		if (x >= 0) {
			this.setCParam(11, 0);
			this.addCParam(10, 1);
			lw_gain = x;
		} else {
			this.addCParam(11, lw_gain);
			lw_gain = 0;
		}
	}
	this.stamCapAdj();
  this.milkSpeedAdj();
  $gameSystem._rc_LewdEXP += amount;
};

Game_Actor.prototype.stamCapAdj = function() {
  var lv = this.lewdlevel;
  var softcap = [0, 30, 60];
  var e = [4, 2, 1];
  var r = 0;
  if (lv > softcap[2]) {
    r = softcap[1] * e[0]; // 30 * 4 = 120
    r += (softcap[2] - softcap[1]) * e[1]; // (60 - 30) * 2 = 60
    r += (lv - softcap[2]) * e[2]; // (61 - 60) * 1 = 1
  }
  else if (lv > softcap[1]) {
    r = softcap[1] * e[0];
    r += (lv - softcap[1]) * e[1];
  }
  else {
    r = lv * e[0];
  }

	var p = 100 + r;
	this.setCParam(9, p);
};

Game_Actor.prototype.milkSpeedAdj = function() {
  var lv = this.lewdlevel;
  var softcap = [0, 30, 60];
  var e = [0.1, 0.05, 0.02];
  var r = 0;
  if (lv > softcap[2]) {
    r = softcap[1] * e[0]; // 30 * 4 = 120
    r += (softcap[2] - softcap[1]) * e[1]; // (60 - 30) * 2 = 60
    r += (lv - softcap[2]) * e[2]; // (61 - 60) * 1 = 1
  }
  else if (lv > softcap[1]) {
    r = softcap[1] * e[0];
    r += (lv - softcap[1]) * e[1];
  }
  else {
    r = lv * e[0];
  }

	var p = 1 + Math.floor(r);
	this.setCParam(15, p);
};

Game_Actor.prototype.milkSizeAdj = function() {
  var milk_size = 0;
  var threshold = [];
  threshold[1] = Math.ceil(this.mmilk * 0.25);
  threshold[2] = Math.ceil(this.mmilk * 0.5);
  threshold[3] = Math.ceil(this.mmilk * 0.75);

  if (this.milk >= threshold[1]) {milk_size = 1};
  if (this.milk >= threshold[2]) {milk_size = 2};
  if (this.milk >= threshold[3]) {milk_size = 3};

  this.setCParam(7, milk_size);
};

Game_Actor.prototype.resetMilk = function() {
  this.setCParam(5, 0);
  this.milkSizeAdj();
};

Game_Actor.prototype.milkMaxAdj = function() {
  var b = this.babies * 50;
  var max = 250 + b;
  this.setCParam(6, max);

  if (this.milk > this.mmlik) this.setCParam(5, this.mmilk);
};

Game_Actor.prototype.jumpstartHorny = function() {
  if (this.isEquipped($dataWeapons[4])) {
    var j = 20;
    if (this.horny < j) {
      this.setCParam(12, j);
    }
  }
};

// ========================================================================================
// Game_Enemy
// ========================================================================================

Game_Enemy.prototype.paramBase = function(paramId) {
  var multiplier = $gameSystem._difficulty == "Safe" ? 0.85 : 1;
  switch (paramId) {
    case 0:
    case 2:
    case 4:
      var p = this.enemy().params[paramId];
      var multi = Math.floor(p * multiplier);
      return multi;
    default:
      return this.enemy().params[paramId];
      break;
  }
};

// ========================================================================================
// Game_Party
// ========================================================================================
Micoomer.GamePartySetup = Game_Party.prototype.initialize;
Game_Party.prototype.initialize = function() {
	Micoomer.GamePartySetup.call(this);
	this._remnants = 0;
};

Game_Party.prototype.remnants = function() {
	return this._remnants;
};

Game_Party.prototype.gainRemnants = function(amount) {
    this._remnants = (this._remnants + amount).clamp(0, this.maxRemnants());
    if (amount > 0) {$gameSystem._rc_BRearned += amount};
};

Game_Party.prototype.loseRemnants = function(amount) {
    this.gainRemnants(-amount);
    $gameSystem._rc_BRspent += amount;
};

Game_Party.prototype.maxRemnants = function() {
    return 9999;
};

Game_Party.prototype.recoverAP = function(amount) {
	$gameSystem._partyAPCur = ($gameSystem._partyAPCur + amount).clamp(0, $gameSystem._partyAPMax);
};

Game_Party.prototype.recoverAllStamina = function() {
	var party = this.size();
	var actor = this.members();
	for (var i = 0; i < party; i++) {
		actor[i].recoverStamina(actor[i].maxstam);
	}
};

Game_Party.prototype.gainGold = function(amount) {
    this._gold = (this._gold + amount).clamp(0, this.maxGold());
    if (amount > 0) {$gameSystem._rc_MoneyEarned += amount};
};

Micoomer.reserveTransfer = Game_Player.prototype.reserveTransfer;
Game_Player.prototype.reserveTransfer = function(mapId, x, y, d, fadeType) {
  Micoomer.reserveTransfer.call(this, mapId, x, y, d, fadeType);
  $gameTemp._alerted = false;
};

// ========================================================================================
// Encounter SHIT
// ========================================================================================
Scene_Map.prototype.checkBoiledPrawn = function() {
  if ($gamePlayer.canMove()) {
    $gamePlayer.setOpacity(this.checkBoiledPrawnOpc());
    if ($gameSystem._boiledPrawnTimer > 0) {$gameSystem._boiledPrawnTimer--};
    if ($gameSystem._boiledPrawnTimer == 0) {
      $gameSystem._boiledPrawnBuff = false;
      $gamePlayer.setOpacity(255);
    }
  }
};

Scene_Map.prototype.checkBoiledPrawnOpc = function() {
  if ($gamePlayer.canMove()) return 150;
  return 255;
};

Game_Interpreter.prototype.encounterReaction = function() {
  var lead = $gameVariables.value(533);
  switch (lead) {
    case 0:
      AudioManager.playSe({name:"[SE] Marie_Eh (Shock)",volume:100,pitch:100});
      break;
    case 1:
      AudioManager.playSe({name:"[SE] Elucia_Eh 3",volume:100,pitch:100});
      break;
  }
};

// ========================================================================================
// ACTOR WINDOW
// ========================================================================================

Scene_Battle.prototype.createActorCommandWindow = function() {
    this._actorCommandWindow = new Window_ActorCommand();
    this._actorCommandWindow.setHandler('skill',  this.commandSkill.bind(this));
    this._actorCommandWindow.setHandler('item',   this.commandItem.bind(this));
    this._actorCommandWindow.setHandler('pass',   this.commandPass.bind(this));
    this._actorCommandWindow.setHandler('pagedown',   this.commandPass.bind(this));
    this.addWindow(this._actorCommandWindow);
};

Window_ActorCommand.prototype.makeCommandList = function() {
    if (this._actor) {
        this.addSkillCommands();
        this.addItemCommand();
        this.addPassCommand();
    }
};

Window_ActorCommand.prototype.addSkillCommands = function() {
  var name = $dataSystem.skillTypes[1];
  this.addCommand(name, 'skill', this._actor.canSkill(), 1);
};

Window_ActorCommand.prototype.addItemCommand = function() {
    this.addCommand(TextManager.item, 'item', this._actor.canItem());
};

Window_ActorCommand.prototype.addPassCommand = function() {
  this.addCommand('Pass', 'pass', this._actor.canPass());
};

Scene_Battle.prototype.commandPass = function() {
  var skill = $dataSkills[2];
  if (BattleManager.actor().meetsSkillConditionsEval(skill)) {
    var action = BattleManager.inputtingAction();
    action.setSkill(skill.id);
    BattleManager.actor().setLastBattleSkill(skill);
    this.onSelectAction();
  };
};
