//=============================================================================
// MCM_SceneMenu.js
//=============================================================================

/*:
 * @plugindesc
 * @author micoomer
 *
 */


Window_MenuCommand.prototype.maxCols = function() {
  return this.maxItems();
};

Window_MenuCommand.prototype.numVisibleRows = function() {
  return 1;
};

Window_MenuCommand.prototype.windowWidth = function() {
    return 90 * this.maxCols();
};

Window_MenuCommand.prototype.itemWidth = function() {
    return 90;
};

Window_MenuStatus.prototype.windowWidth = function() {
  var m = Math.min(Math.max($gameParty.members().length,0),5);
  var w = 256 * m;
  return w;
};

Window_MenuStatus.prototype.itemWidth = function() {
    return 256;
};

Window_MenuStatus.prototype.numVisibleRows = function() {
  return 1;
};

Window_MenuStatus.prototype.maxCols = function() {
  return this.maxItems();
};

Window_MenuStatus.prototype.drawItem = function(index) {
};

Window_MenuCommand.prototype.addOriginalCommands = function() {
  this.addCommand("Record", 'record', true);
};

//=============================================================================
// ** Scene Menu
//=============================================================================

Scene_Menu.prototype.createCommandWindow = function() {
    this._commandWindow = new Window_MenuCommand(0, 0);
    this._commandWindow.setHandler('item',      this.commandItem.bind(this));
    this._commandWindow.setHandler('skill',     this.commandPersonal.bind(this));
    this._commandWindow.setHandler('equip',     this.commandPersonal.bind(this));
    this._commandWindow.setHandler('status',    this.commandPersonal.bind(this));
    this._commandWindow.setHandler('record',    this.commandRecord.bind(this));
    this._commandWindow.setHandler('options',   this.commandOptions.bind(this));
    this._commandWindow.setHandler('save',      this.commandSave.bind(this));
    this._commandWindow.setHandler('cancel',    this.popScene.bind(this));
    this.addWindow(this._commandWindow);
};

Scene_Menu.prototype.commandRecord = function() {
    SceneManager.push(Scene_Record);
};

Scene_Menu.prototype.createStatusWindow = function() {
  var w = 256 * this.maxMembers();
  this._statusWindow = new Window_MenuStatus(0, 0);
  this.addWindow(this._statusWindow);
};

Micoomer.Scene_MenuCreate = Scene_Menu.prototype.create;
Scene_Menu.prototype.create = function() {
	Micoomer.Scene_MenuCreate.call(this);

  this._commandWindow.contentsOpacity = 0;
  this._commandWindow.x = 620 - (45 * this._commandWindow._list.length);
  this._commandWindow.y = 645;
  this._statusWindow.contentsOpacity = 0;
  this._statusWindow.x = 640 - (this._statusWindow.width) + (128 * this.maxMembers());
  this._statusWindow.y = -100;

	this.loadBitmaps();
	this.createField();
	this.createSprites();
};

Scene_Menu.prototype.loadBitmaps = function() {
	this._layImg = ImageManager.loadBackground("UI_MainMenu");
};

Scene_Menu.prototype.createField = function() {
  this._field = new Sprite();
	this.addChild(this._field);
};

Scene_Menu.prototype.createSprites = function() {
  this.create_Bust();
	this.create_LayTop();
  this.create_LayBot();
  this.create_Status();
  this.create_Select();
	this.create_Com();
	this.create_Currency();
};

//==============================
// * create Sprites
//==============================
Scene_Menu.prototype.create_LayTop = function() {
  this._layTop = new Sprite(this._layImg);
	this._field.addChild(this._layTop);
  this._layTop.setFrame(0,0,1280,10);
};

Scene_Menu.prototype.create_LayBot = function() {
  this._layBot = new Sprite(this._layImg);
  this._layBot.y = 650;
	this._field.addChild(this._layBot);
  this._layBot.setFrame(0,10,1280,56);
};

Scene_Menu.prototype.create_Bust = function() {
	this._bust = [];
	for (var i = 0; i < this.maxMembers(); i++) {
    this._bust[i] = new Menu_Bust(i,$gameParty.members()[i],this.maxMembers());
		this._field.addChild(this._bust[i]);
	}
};

Scene_Menu.prototype.create_Status = function() {
	this._status = [];
	for (var i = 0; i < this.maxMembers(); i++) {
    this._status[i] = new Menu_Status(i,$gameParty.members()[i],this.maxMembers());
		this._field.addChild(this._status[i]);
	}
};

Scene_Menu.prototype.create_Com = function() {
	this._comField = new Sprite();
	this._field.addChild(this._comField);

  this._acCom = [];
  this._inCom = [];
  this._comPosX = [];
  this._comPosY = [673, 667];
  this._comList = this._commandWindow._list;
	for (var i = 0; i < this._comList.length; i++) {
    this._comPosX[i] = 686 - (45 * this._comList.length) + (90 * i);
    var frame = (62 * i);
		this._acCom[i] = new Sprite(this._layImg);
		this._acCom[i].x = this._comPosX[i];
		this._acCom[i].y = 673;
		this._acCom[i].anchor.x = 0.5;
		this._acCom[i].anchor.y = 0.5;
    this._acCom[i].setFrame(frame,326,62,50);
	  this._comField.addChild(this._acCom[i]);

    this._inCom[i] = new Sprite(this._layImg);
		this._inCom[i].x = this._comPosX[i];
		this._inCom[i].y = 673;
		this._inCom[i].anchor.x = 0.5;
		this._inCom[i].anchor.y = 0.5;
    this._inCom[i].setFrame(frame,376,62,50);
	  this._comField.addChild(this._inCom[i]);

	}
};

Scene_Menu.prototype.create_Select = function() {
  this._select = new Sprite(this._layImg);
  this._select.anchor.x = 0.5;
  this._select.anchor.y = 0.5;
  this._select.x = 640;
  this._select.y = 681;
	this._field.addChild(this._select);
  this._select.setFrame(720,251,88,35);
};

Scene_Menu.prototype.create_Currency = function() {

  // Money
  this._money = new Sprite(new Bitmap(260,32));
	this._money.x = 875;
	this._money.y = 665;
	this._money.bitmap.fontSize = 20;
	var text = $gameParty.gold();
	this._money.bitmap.drawText(text,0,0,260,32,"right");
	this._field.addChild(this._money);

  // Remnants
  this._rem = new Sprite(new Bitmap(260,32));
	this._rem.x = this._money.x + 120;
	this._rem.y = this._money.y;
	this._rem.bitmap.fontSize = 20;
	var text = $gameParty.remnants();
	this._rem.bitmap.drawText(text,0,0,260,32,"right");
	this._field.addChild(this._rem);
};

Scene_Menu.prototype.maxMembers = function() {
  return Math.min(Math.max($gameParty.members().length,0),5);
};

Scene_Menu.prototype.isComEnabled = function(index) {
	if (index != this._commandWindow._index) {return false};
	return true;
};

Scene_Menu.prototype.updateCommands = function() {
  for (var i = 0; i < this._acCom.length; i++) {
		if (this.isComEnabled(i)) {
			if (this._commandWindow.isCurrentItemEnabled()) {
        this._inCom[i].visible = false;
        this._select.x = globMoveTo(this._select.x, this._inCom[i].x, 3);

      }
		} else {
      this._inCom[i].visible = true;
		}
    this._acCom[i].visible = !this._inCom[i].visible;
    var yi = this._acCom[i].visible ? 1 : 0;
    this._acCom[i].y = globMoveTo(this._acCom[i].y, this._comPosY[yi], 6);
	}
};

//==============================
// * update
//==============================
Micoomer.Scene_MenuUpdate = Scene_Menu.prototype.update;
Scene_Menu.prototype.update = function() {
  Micoomer.Scene_MenuUpdate.call(this)
	if (this._acCom) {this.updateCommands()};
};

//=============================================================================
// * Menu_Bust
//=============================================================================
function Menu_Bust() {
    this.initialize.apply(this, arguments);
};

Menu_Bust.prototype = Object.create(Sprite.prototype);
Menu_Bust.prototype.constructor = Menu_Bust;

Menu_Bust.prototype.initialize = function(index,actor,maxmembers) {
    Sprite.prototype.initialize.call(this);

  this._index = index;
	this._actor = actor;
	this._maxMembers = maxmembers;
	this._wait = 5 + 10 * this._index;
  this.loadBitmaps();
	this.createLayout();
	this.createCharaters();
};

Menu_Bust.prototype.loadBitmaps = function() {
  this._layImg = ImageManager.loadBackground("UI_MainMenu");
};

Menu_Bust.prototype.posX = function() {
	var space = 640 - (128 * $gameParty.members().length) + (256 * this._index);
	return space;
};

Menu_Bust.prototype.createLayout = function() {
  this._layout = new Sprite(this._layImg);
  this._layout.setFrame(0, 66, 720, 260);
	this._layout.anchor.x = 0.5;
	this._layout.x = this.posX();
	this._layout.y = 360;
	this._layout.opacity = 0;
  this._layout.rotation = -90 * Math.PI / 180;
	this.addChild(this._layout);
};

Menu_Bust.prototype.createCharaters = function() {
  this._char = new MSprite2(this._actor._actorId);
	this._char.anchor.x = 0.5;
	this._char.x = this._layout.x;
	this._char.opacity = 0;
	this.addChild(this._char);
};

Menu_Bust.prototype.update = function() {
    Sprite.prototype.update.call(this);

  if (this._wait > 0) {this._wait--; return};
	this._layout.opacity += 10;
	this._char.opacity = this._layout.opacity;
};

//=============================================================================
// * Menu_Status
//=============================================================================
function Menu_Status() {
    this.initialize.apply(this, arguments);
};

Menu_Status.prototype = Object.create(Sprite.prototype);
Menu_Status.prototype.constructor = Menu_Status;

Menu_Status.prototype.initialize = function(index,actor,maxmembers) {
    Sprite.prototype.initialize.call(this);

  this._index = index;
	this._actor = actor;
	this._maxMembers = maxmembers;
	this._wait = 5 + 10 * this._index;
	this.x = 0;
	this.opacity = 0;
	this.loadBitmaps();
};

Menu_Status.prototype.loadBitmaps = function() {
  this._layImg = ImageManager.loadBackground("UI_MainMenu");
};

Menu_Status.prototype.posX = function() {
	var space = 768 - (128 * $gameParty.members().length) + (256 * this._index);
	return space;
};

Menu_Status.prototype.createSprites = function() {
	this.createLayoutStatus();
	this.createMilkMeter();
	this.createMilkCap();
	this.createMilkBottle();
	this.createHPMeter();
	this.createFtgMeter();
	this.createPregMeter();
	this.createHP();
	this.createExtra();
	this.createName();
	this.createLV();
};

Menu_Status.prototype.createLayoutStatus = function() {
   this._layout = new Sprite(this._layImg);
	 this._layout.x = this.posX() - 105;
	 this._layout.y = 522;
   this._layout.setFrame(720,66,220,110);
	 this.addChild(this._layout);
};

Menu_Status.prototype.createName = function() {
	this._name = new Sprite(new Bitmap(120,32));
	this._name.x = this.posX() - 116;
	this._name.y = 10;
  this._name.bitmap.fontFace = 'FredokaBold';
	this._name.bitmap.fontSize = 30;
	this._name.bitmap.drawText(this._actor.name(),0,0,120,32,"left");
	this.addChild(this._name);

};

Menu_Status.prototype.createLV = function() {
	this._lv = new Sprite(new Bitmap(120,32));
	this._lv.x = this._name.x;
	this._lv.y = this._name.y + 30;
	this._lv.text = "Lv. " + this._actor.level;
	this._lv.bitmap.fontSize = 14;
  this._lv.bitmap.fontFace = 'FredokaBold';
	this._lv.bitmap.drawText(this._lv.text,0,0,120,32,"left");
	this.addChild(this._lv);

};

Menu_Status.prototype.createHP = function() {
	this._hp = new Sprite(new Bitmap(120,32));
	this._hp.x = this._layout.x + 82;
	this._hp.y = this._layout.y + 4;
	this._hp.text = this._actor.hp;
	this._hp.bitmap.fontSize = 14;
	this._hp.bitmap.fontFace = 'FredokaSemi';
	this._hp.bitmap.textColor = '#f57ac8';
	this._hp.bitmap.drawText(this._hp.text,0,0,120,32,"right");
	this.addChild(this._hp);
};

Menu_Status.prototype.createExtra = function() {
	this._ftg = new Sprite(new Bitmap(120,32));
	this._ftg.x = this._layout.x + 82;
	this._ftg.y = this._layout.y + 34;
	this._ftg.text = (this._actor.stam / this._actor.maxstam) * 100;
	this._ftg.text = Math.ceil(this._ftg.text);
	this._ftg.bitmap.fontSize = 14;
	this._ftg.bitmap.fontFace = 'FredokaSemi';
  this._ftg.bitmap.textColor = '#f57ac8';
	this._ftg.bitmap.drawText(this._ftg.text + "%",0,0,120,32,"right");
	this.addChild(this._ftg);

  this._pregNum = new Sprite(new Bitmap(120,32));
	this._pregNum.x = this._layout.x + 82;
	this._pregNum.y = this._layout.y + 64;
	this._pregNum.text = (this._actor.pregnancy / 1000) * 100;
	this._pregNum.text = Math.ceil(this._pregNum.text);
	this._pregNum.bitmap.fontSize = 14;
  this._pregNum.bitmap.fontFace = 'FredokaSemi';
	this._pregNum.bitmap.textColor = '#f57ac8';
	this._pregNum.bitmap.drawText(this._pregNum.text + "%",0,0,120,32,"right");
	this.addChild(this._pregNum);
};

Menu_Status.prototype.createHPMeter = function() {
  this._hpMeter = new Sprite(this._layImg);
	this._hpMeterData = [136,6,940,66];
	this._hpMeter.x = this._layout.x + 67;
	this._hpMeter.y = this._layout.y + 31;
	this.updateMeter(this._hpMeter,this._hpMeterData,this._actor.hp,this._actor.mhp);
	this.addChild(this._hpMeter);
};

Menu_Status.prototype.createFtgMeter = function() {
  this._ftgMeter = new Sprite(this._layImg);
	this._ftgMeterData = [136,6,940,72];
	this._ftgMeter.x = this._layout.x + 67;
	this._ftgMeter.y = this._layout.y + 61;
	this.updateMeter(this._ftgMeter,this._ftgMeterData,this._actor.stam,this._actor.maxstam);
	this.addChild(this._ftgMeter);
};

Menu_Status.prototype.createPregMeter = function() {
	this._pregMeter = new Sprite(this._layImg);
	this._pregMeterData = [136,6,940,78];
	this._pregMeter.x = this._ftgMeter.x;
	this._pregMeter.y = this._ftgMeter.y + 30;
	this.updateMeter(this._pregMeter,this._pregMeterData,this._actor.pregnancy,1000);
	this.addChild(this._pregMeter);
};

Menu_Status.prototype.createMilkMeter = function() {
	this._milkMeter = new Sprite(this._layImg);
	this._milkMeterData = [44,31,790,176];
	this._milkMeter.x = this._layout.x + 8;
	this._milkMeter.y = this._layout.y + 91;
	this._milkMeter.rotation = -90 * Math.PI / 180;
	this.updateMeter(this._milkMeter,this._milkMeterData,this._actor.milk,this._actor.mmilk);
	this.addChild(this._milkMeter);
};

Menu_Status.prototype.createMilkCap = function() {
   this._milkCap = new Sprite(this._layImg);
	 this._milkCap.x = this._layout.x + 6;
	 this._milkCap.y = this._layout.y + 18;
   this._milkCap.setFrame(755,176,35,75);
	 this.addChild(this._milkCap);
	if (this._actor.milk !== this._actor.mmilk) {this._milkCap.visible = false};
};

Menu_Status.prototype.createMilkBottle = function() {
   this._milkBottle = new Sprite(this._layImg);
	 this._milkBottle.x = this._layout.x + 6;
	 this._milkBottle.y = this._layout.y + 18;
   this._milkBottle.setFrame(720,176,35,75);
	 this.addChild(this._milkBottle);
};

//==============================
// * update Meter
//==============================
Menu_Status.prototype.updateMeter = function(sprite,data,v1,v2) {
	var cw = data[0] * v1 / v2;
  sprite.setFrame(data[2],data[3],cw,data[1]);
};


//==============================
// * Update
//==============================
Menu_Status.prototype.update = function() {
  Sprite.prototype.update.call(this);
	if (!this._actor) {return}
    if (!this._hp) {
      if (this._layImg.isReady()) {this.createSprites()} else {return};
	  }
	  if (this._wait > 0) {this._wait--;return}
	  this.opacity += 10;
};

//=============================================================================
// ** Window Menu Command
//=============================================================================

Window_MenuCommand.prototype.processCursorMove = function() {
    if (this.isCursorMovable()) {
        var lastIndex = this.index();
        if (Input.isRepeated('right')) {
            this.cursorRight();
        };
        if (Input.isRepeated('left')) {
            this.cursorLeft();
        };
        if (this.index() !== lastIndex) {
            SoundManager.playCursor();
        };
    };
};

Window_MenuStatus.prototype.processCursorMove = function() {
    if (this.isCursorMovable()) {
        var lastIndex = this.index();
        if (Input.isRepeated('right')) {
            this.cursorRight();
        };
        if (Input.isRepeated('left')) {
            this.cursorLeft();
        };
        if (this.index() !== lastIndex) {
            SoundManager.playCursor();
        };
    };
};
