//=============================================================================
// MCM_SceneStatus.js
//=============================================================================

/*:
 * @plugindesc
 * @author micoomer
 *
 */

	Window_Gold.prototype.windowWidth = function() {return 200};

	Window_Gold.prototype.windowHeight = function() {return 100};

	Window_ShopBuy.prototype.windowWidth = function() {return 369};

	Window_ShopBuy.prototype.windowHeight = function() {return 300};

	Window_ShopNumber.prototype.windowWidth = function() {return 360};

	Window_ShopNumber.prototype.windowHeight = function() {return 300};

	//-----------------------------------------------------------------------------
	// Window_GoldShop
	//-----------------------------------------------------------------------------

	function Window_GoldShop() {
	    this.initialize.apply(this, arguments);
	}

	Window_GoldShop.prototype = Object.create(Window_Base.prototype);
	Window_GoldShop.prototype.constructor = Window_GoldShop;

	Window_GoldShop.prototype.initialize = function(x, y) {
	    var width = 200;
	    var height = 100;
	    Window_Base.prototype.initialize.call(this, x, y, width, height);
			this.contents.fontSize = 20;
	    this.refresh();
	};

	Window_GoldShop.prototype.refresh = function() {
	    this.contents.clear();
	    this.drawText(this.value(), -100, 0, 200, 'right');
	};

	Window_GoldShop.prototype.value = function() {
	    return $gameParty.gold();
	};

	//-----------------------------------------------------------------------------
	// Window_ShopStatus
	//
	// The window for displaying number of items in possession and the actor's
	// equipment on the shop screen.

	function Window_HeldWindow() {
	    this.initialize.apply(this, arguments);
	}

	Window_HeldWindow.prototype = Object.create(Window_Base.prototype);
	Window_HeldWindow.prototype.constructor = Window_HeldWindow;

	Window_HeldWindow.prototype.initialize = function(x, y, width, height) {
	    Window_Base.prototype.initialize.call(this, x, y, width, height);
	    this._item = null;
			this.contents.textColor = '#ffffff';
	    this.refresh();
	};

	Window_HeldWindow.prototype.refresh = function() {
	    this.contents.clear();
	    if (this._item) {
	        this.drawPossession();
	    }
	};

	Window_HeldWindow.prototype.setItem = function(item) {
	    this._item = item;
	    this.refresh();
	};

	Window_HeldWindow.prototype.drawPossession = function() {
			var text = $gameParty.numItems(this._item) + ' / ' + Yanfly.Param.MaxItem;
	    this.drawText(text, -80, -8, 200, 'right');
	};

//=============================================================================
// ** Scene Shop
//=============================================================================

Scene_Shop.prototype.create = function() {
    Scene_MenuBase.prototype.create.call(this);
    this.createHelpWindow();
		this._helpWindow.x = 333;
		this._helpWindow.y = 576;
		this._opacity = 0;

    this.createGoldWindow();
    this.createNumberWindow();
		this.createHeldWindow();
    this.createBuyWindow();

		// Sprite
		this.loadBitmaps();
		this.createSprites();

		// activate
		this.activateBuyWindow();
};

Scene_Shop.prototype.createGoldWindow = function() {
		var x = 965;
		var y = 104;
    this._goldWindow = new Window_GoldShop(x, y);
//		this._goldWindow.opacity = 0;
    this.addWindow(this._goldWindow);
};

Scene_Shop.prototype.createNumberWindow = function() {
    var x = 480;
    var y = 255;
    this._numberWindow = new Window_ShopNumber(x, y, 200);
    this._numberWindow.hide();
    this._numberWindow.setHandler('ok',     this.onNumberOk.bind(this));
    this._numberWindow.setHandler('cancel', this.onNumberCancel.bind(this));
    this.addWindow(this._numberWindow);
		this._numberWindow.opacity = 0;
};

Scene_Shop.prototype.createBuyWindow = function() {
    var x = 453;
    var y = 122;
    this._buyWindow = new Window_ShopBuy(x, y, 370, this._goods);
    this._buyWindow.setHelpWindow(this._helpWindow);
		this._buyWindow.setStatusWindow(this._heldWindow);
    this._buyWindow.hide();
    this._buyWindow.setHandler('ok',     this.onBuyOk.bind(this));
    this._buyWindow.setHandler('cancel', this.onBuyCancel.bind(this));
    this.addWindow(this._buyWindow);
		this._buyWindow.opacity = 0;
};

Scene_Shop.prototype.createHeldWindow = function() {
    var x = 657;
    var y = 473;
    this._heldWindow = new Window_HeldWindow(x, y, 160, 60);
    this.addWindow(this._heldWindow);
		this._heldWindow.opacity = 0;
};

Scene_Shop.prototype.activateBuyWindow = function() {
    this._buyWindow.setMoney(this.money());
    this._buyWindow.show();
    this._buyWindow.activate();
};

Scene_Shop.prototype.onBuyOk = function() {
    this._item = this._buyWindow.item();
    this._buyWindow.hide();
    this._numberWindow.setup(this._item, this.maxBuy(), this.buyingPrice());
    this._numberWindow.show();
    this._numberWindow.activate();
};

Scene_Shop.prototype.onBuyCancel = function() {
    this.popScene();
};

Scene_Shop.prototype.onNumberOk = function() {
    SoundManager.playShop();
		this.doBuy(this._numberWindow.number());
    this.endNumberInput();
    this._goldWindow.refresh();
};

Scene_Shop.prototype.onNumberCancel = function() {
    SoundManager.playCancel();
    this.endNumberInput();
};

Scene_Shop.prototype.endNumberInput = function() {
    this._numberWindow.hide();
		this.activateBuyWindow();
};

//////////////////////////////////////////////////////////////////////////////

var _mcm_shop_createBackground = Scene_Shop.prototype.createBackground;
Scene_Shop.prototype.createBackground = function() {
	_mcm_shop_createBackground.call(this);
	this._field = new Sprite();
	this.addChild(this._field);
};

Scene_Shop.prototype.loadBitmaps = function() {
	this._layImg = ImageManager.loadBackground('UI_ShopMenu');
};

Scene_Shop.prototype.createSprites = function() {
	this.createSprite_Title();
	this.createSprite_List();
	this.createSprite_Money();
	this.createSprite_Help();
	this.createSprite_ShopNumber();
};

Scene_Shop.prototype.createSprite_Title = function() {
	this._lay_title = new Sprite(this._layImg);
	this._lay_title.setFrame(400,170,480,100);
	this._field.addChild(this._lay_title);
};

Scene_Shop.prototype.createSprite_List = function() {
	this._lay_list = new Sprite(this._layImg);
	this._lay_list.x = 440;
	this._lay_list.y = 96;
	this._lay_list.setFrame(0,0,400,440);
	this._field.addChild(this._lay_list);
};

Scene_Shop.prototype.createSprite_Help = function() {
	this._lay_help = new Sprite(this._layImg);
	this._lay_help.x = 340;
	this._lay_help.y = 550;
	this._lay_help.setFrame(0,440,600,130);
	this._field.addChild(this._lay_help);
};

Scene_Shop.prototype.createSprite_Money = function() {
	this._lay_money = new Sprite(this._layImg);
	this._lay_money.x = 855;
	this._lay_money.y = 120;
	this._lay_money.setFrame(400,130,250,40);
	this._field.addChild(this._lay_money);
};

Scene_Shop.prototype.createSprite_ShopNumber = function() {
	this._lay_number = new Sprite(this._layImg);
	this._lay_number.x = 430;
	this._lay_number.y = 258;
	this._lay_number.visible = false;
	this._lay_number.setFrame(400,0,420,130);
	this._field.addChild(this._lay_number);
};

Scene_Shop.prototype.updateShopNumber = function() {
	this._lay_number.visible = this._numberWindow.active;
};

var _mcm_shop_update = Scene_Shop.prototype.update;
Scene_Shop.prototype.update = function() {
	_mcm_shop_update.call(this);
	this.updateShopNumber();
};
