//=============================================================================
// mi_TravelMenu.js
//=============================================================================


/*:
 * @plugindesc
 * @author micoomer
 *
 */


function Scene_Travel() {
    this.initialize.apply(this, arguments);
}

Scene_Travel.prototype = Object.create(Scene_MenuBase.prototype);
Scene_Travel.prototype.constructor = Scene_Travel;

Scene_Travel.prototype.initialize = function() {
    Scene_MenuBase.prototype.initialize.call(this);
};

Scene_Travel.prototype.create = function() {
  Scene_MenuBase.prototype.create.call(this);
  this.loadBitmaps();

  this.createWindow_Panel();

  this.createLayout();
  this.createMapCursor();

  this._w_panel.activate();
  this._w_panel.show();
  this._w_panel.select(0);
};

Scene_Travel.prototype.loadBitmaps = function() {
  this._spr = ImageManager.loadBackground("UI_Travel");
};

Scene_Travel.prototype.createWindow_Panel = function() {
    var wx = 83;
    var wy = 117;
    var ww = 340;
    var wh = 360;
    this._w_panel = new Window_TravelMenu(wx, wy, ww, wh);
    this._w_panel.setHandler('ok',     this.onTravelOk.bind(this));
    this._w_panel.setHandler('cancel', this.onTravelCancel.bind(this));
    this.addWindow(this._w_panel);
};

Scene_Travel.prototype.onTravelOk = function() {
  var item = this._w_panel.item();
  SceneManager.pop();
  this.fadeOutAll();
  $gamePlayer.reserveTransfer(item.mapId, item.mapX, item.mapY, 2, 0);
};

Scene_Travel.prototype.onTravelCancel = function() {
  SceneManager.pop();
};

Micoomer.travelBG = Scene_Travel.prototype.createBackground;
Scene_Travel.prototype.createBackground = function() {
	Micoomer.travelBG.call(this);
	this._field = new Sprite();
	this.addChild(this._field);
};

Scene_Travel.prototype.createLayout = function() {
  this._bg = [];
	this._bg = new Sprite(this._spr);
	this._field.addChild(this._bg);
};

Scene_Travel.prototype.createMapCursor = function() {
	this._mapcur = new Sprite(ImageManager.loadSystem("PinPoint"));
  this._mapcur.anchor.x = 0.5;
  this._mapcur.anchor.y = 0.5;
  this._mapcur.x = 873;
  this._mapcur.y = 175;
  this._field.addChild(this._mapcur);
};

Scene_Travel.prototype.updateCursor = function() {
  var itemX = this._w_panel.item().corX;
  var itemY = this._w_panel.item().corY;
	this._mapcur.x = globMoveTo(this._mapcur.x, itemX, 6);
  this._mapcur.y = globMoveTo(this._mapcur.y, itemY, 6);
};

Scene_Travel.prototype.update = function() {
  Scene_MenuBase.prototype.update.call(this);
  this.updateCursor();
};


//=============================================================================
// * Window
//=============================================================================
function Window_TravelMenu() {
    this.initialize.apply(this, arguments);
}

Window_TravelMenu.prototype = Object.create(Window_Selectable.prototype);
Window_TravelMenu.prototype.constructor = Window_TravelMenu;

Window_TravelMenu.prototype.initialize = function(x, y, width, height) {
    Window_Selectable.prototype.initialize.call(this, x, y, width, height);
    this._data = [];
    this.refresh();
    this.resetScroll();
};

Window_TravelMenu.prototype.maxCols = function() {
    return 1;
};

Window_TravelMenu.prototype.maxItems = function() {
    return this._data ? this._data.length : 1;
};

Window_TravelMenu.prototype.item = function() {
    return this._data && this.index() >= 0 ? this._data[this.index()] : null;
};

Window_TravelMenu.prototype.isCurrentItemEnabled = function() {
    return this.isEnabled(this._data[this.index()]);
};

Window_TravelMenu.prototype.isEnabled = function(item) {
    return $gameSystem._travelLocation[item.type][item.switch];
};

Window_TravelMenu.prototype.makeItemList = function() {
  var tempData = Micoomer.travelArray;
  var dat = tempData.slice();
  for (var i = tempData.length - 1; i >= 0; i--) {
    if (!this.isEnabled(dat[i])) {
      dat.splice(i,1);
    }
  }
  this._data = dat;
};

Window_TravelMenu.prototype.drawItem = function(index) {
    var i = this._data[index];
    if (i) {
        var rect = this.itemRect(index);
        rect.width -= this.textPadding();
        this.changePaintOpacity(this.isEnabled(i));
        this.drawIcon(i.iconIndex, rect.x, rect.y + 2);
        this.drawText(i.name, rect.x + 38, rect.y, rect.width);
    }
};

Window_TravelMenu.prototype.refresh = function() {
    this.makeItemList();
    this.createContents();
    this.drawAllItems();
};
