//=============================================================================
// MOG_ActionName.js
//=============================================================================

/*:
 * @plugindesc (v1.3) Apresenta uma janela com nome da ação.
 * @author Moghunter
 *
 * @help
 * =============================================================================
 * +++ MOG - Action Name (v1.3) +++
 * By Moghunter
 * https://atelierrgss.wordpress.com/
 * =============================================================================
 * Apresenta uma janela com nome da ação.
 * Será necessário ter o arquivo. (img/system/)
 *
 * ActionName.png
 *
 * =============================================================================
 * Desativar o Nome
 * =============================================================================
 * Inicialmente todas as ações terão o nome ativado, no caso de querer desativar
 * algum nome, utilize o comentário abaixo na caixa de notas do Item/Habilidade.
 *
 * Disable Name
 *
 * =============================================================================
 * HISTÓRICO
 * =============================================================================
 * (v1.3) - Correção da transparência inicial da imagem.
 * (v1.2) - Melhoria na codificação e na compatibilidade de plugins.
 * (v1.1) - Compatibilidade com o Chrono Engine.
 */

//=============================================================================
// ** PLUGIN PARAMETERS
//=============================================================================
　　var Imported = Imported || {};
　　Imported.MOG_ActionName = true;
　　var Moghunter = Moghunter || {};

  　Moghunter.parameters = PluginManager.parameters('MOG_ActionName');
    Moghunter.skillName_x = Number(Moghunter.parameters['Layout X-Axis'] || 640);
    Moghunter.skillName_y = Number(Moghunter.parameters['Layout Y-Axis'] || 100);
    Moghunter.skillName_name_x = Number(Moghunter.parameters['Name X-Axis'] || 2);
    Moghunter.skillName_name_y = Number(Moghunter.parameters['Name Y-Axis'] || 2);
    Moghunter.skillName_FontSize = Number(Moghunter.parameters['Font Size'] || 20);

//=============================================================================
// ** Game Temp
//=============================================================================

//==============================
// * Initialize
//==============================
var _alias_mog_skillname_initialize = Game_Temp.prototype.initialize
Game_Temp.prototype.initialize = function() {
	_alias_mog_skillname_initialize.call(this);
	this._skillNameData = [false,null];
};

//=============================================================================
// ** Battle Manager
//=============================================================================

//==============================
// ** Start Action
//==============================
var _mog_skillName_bmngr_startAction = BattleManager.startAction;
BattleManager.startAction = function() {
    this.setSkillName();
	_mog_skillName_bmngr_startAction.call(this);
};

//==============================
// * set Skill Name
//==============================
BattleManager.setSkillName = function() {
  if (!this._subject.currentAction() || !this._subject.currentAction().item()) {return};
	var item = this._subject.currentAction().item();
	var notes = item.note.split(/[\r\n]+/);
	var enableName = true;
    notes.forEach(function(note) {
    if (note == "Disable Name" ) {enableName = false};
	},this);
	if (enableName) {$gameTemp._skillNameData = [true,item]};
};

//==============================
// * End Turn
//==============================
var _mog_skillName_bmngr_endAction  = BattleManager.endAction;
BattleManager.endAction = function() {
	_mog_skillName_bmngr_endAction.call(this);
	$gameTemp._skillNameData = [false,null];
};

//=============================================================================
// ** Scene Base
//=============================================================================

//==============================
// ** create Hud Field
//==============================
Scene_Base.prototype.createHudField = function() {
	this._hudField = new Sprite();
	this._hudField.z = 10;
	this.addChild(this._hudField);
};

//==============================
// ** sort MZ
//==============================
Scene_Base.prototype.sortMz = function() {
   this._hudField.children.sort(function(a, b){return a.mz-b.mz});
};

//==============================
// * create Action Name
//==============================
Scene_Base.prototype.createActionName = function() {
	this._spriteSkillName = new SpriteSkillName();
	this._spriteSkillName.mz = 140;
    this._hudField.addChild(this._spriteSkillName);
};

//=============================================================================
// ** SpriteSet Base
//=============================================================================

//==============================
// ** create Hud Field
//==============================
Spriteset_Base.prototype.createHudField = function() {
	this._hudField = new Sprite();
	this._hudField.z = 10;
	this.addChild(this._hudField);
};

//==============================
// ** sort MZ
//==============================
Spriteset_Base.prototype.sortMz = function() {
   this._hudField.children.sort(function(a, b){return a.mz-b.mz});
};

//==============================
// * create Action Name
//==============================
Spriteset_Base.prototype.createActionName = function() {
	this._spriteSkillName = new SpriteSkillName();
	this._spriteSkillName.mz = 130;
    this._hudField.addChild(this._spriteSkillName);
};

//=============================================================================
// ** Scene Battle
//=============================================================================

//==============================
// ** create Spriteset
//==============================
var _mog_actionName_sbattle_createSpriteset = Scene_Battle.prototype.createSpriteset;
Scene_Battle.prototype.createSpriteset = function() {
	_mog_actionName_sbattle_createSpriteset.call(this);
	if (!this._hudField) {this.createHudField()};
    this.createActionName();
	this.sortMz();
};

//=============================================================================
// * Sprite Skill Name
//=============================================================================
function SpriteSkillName() {
    this.initialize.apply(this, arguments);
};

SpriteSkillName.prototype = Object.create(Sprite.prototype);
SpriteSkillName.prototype.constructor = SpriteSkillName;

//==============================
// * Initialize
//==============================
SpriteSkillName.prototype.initialize = function() {
    Sprite.prototype.initialize.call(this);
	this.visible = false;
	this._active = false;
	this._wait = 0;
	this._space = 120;
	this._org = [-170, 0, 60, 230];
	this._phase = 0;
	this._timer = 0;
	this._duration = [15, 50, 15];
    this.loadImages();
	this.createSprites();
};

//==============================
// * Load Images
//==============================
SpriteSkillName.prototype.loadImages = function() {
	this._layoutImg = ImageManager.loadSystem("ActionName");
};

//==============================
// * Item
//==============================
SpriteSkillName.prototype.item = function() {
    return $gameTemp._skillNameData[1];
};

//==============================
// * is Visible
//==============================
SpriteSkillName.prototype.isVisible = function() {
	if (this._wait > 0) {return false};
	if (this._active) {return true};
	return false
};

SpriteSkillName.prototype.createSprites = function() {
	this.createLayout();
	this.createName();
};

//==============================
// * refresh Skill Name
//==============================
SpriteSkillName.prototype.refreshSkillName = function() {
	$gameTemp._skillNameData[0] = false;
	this.opacity = 0;
	this._active = true;
	this._wait = 4;
	this._phase = 0;
	this._timer = this._duration[0];
	this._duration = [10, 58, 12];

	this._name.bitmap.clear();
	if (!this.item()) {return};
	var text = this.item().name;
	this._name.bitmap.drawText(text,0,0,200,32,"center");

	this.resetPosition();
};

SpriteSkillName.prototype.resetPosition = function() {
	this.x = this._org[0];
};

//==============================
// * create Layout
//==============================
SpriteSkillName.prototype.createLayout = function() {
	this._layout = new Sprite(this._layoutImg);
	this._layout.anchor.x = 0.5;
	this._layout.x = Moghunter.skillName_x;
	this._layout.y = Moghunter.skillName_y;
	this._layout.z = 20;
	this.addChild(this._layout);
};

SpriteSkillName.prototype.createName = function() {
  this._name = new Sprite(new Bitmap(200,32));
  this._name.anchor.x = 0.5;
  this._name.x = this._layout.x + 5;
	this._name.y = this._layout.y - 2;
	this._name.bitmap.fontSize = Moghunter.skillName_FontSize;
  this._name.bitmap.fontFace = 'FredokaSemi'
	this._name.bitmap.textColor = '#f57ac8';
	this.addChild(this._name);
};

//==============================
// * Update
//==============================
SpriteSkillName.prototype.updateVisible = function() {
	if (this._wait > 0) {this._wait--};
	if (this.isVisible()) {
	    this.opacity += 20;
	} else {
	    this.opacity -= 20;
	};
};

SpriteSkillName.prototype.checkPhase = function() {
	if (this._timer == 0 && this._phase < 3) {
		this._phase++;
		this._timer = this._duration[this._phase];
	};
	if (this._phase == 2) {
		this._active = false;
	};
	if (this._timer > 0) {this._timer--};
	var p = this._phase + 1;
	this.x = this.sprite_move_to(this.x, this._org[p], this._duration[this._phase]);
};

SpriteSkillName.prototype.sprite_move_to = function(value,real_value,duration) {
	if (value === real_value) {return value};
	var dnspeed = 1 + (Math.abs(value - real_value) / duration);
	if (value > real_value) {value -= dnspeed;
	    if (value < real_value) {value = real_value};}
    else if (value < real_value) {value  += dnspeed;
    	if (value  > real_value) {value  = real_value};
    };
	return Math.floor(value);
};

//==============================
// * Update
//==============================
SpriteSkillName.prototype.update = function() {
    Sprite.prototype.update.call(this);
	if ($gameTemp._skillNameData[0]) {this.refreshSkillName()};
	if (this.opacity > 0 && this._wait == 0) {this.checkPhase()};
	this.updateVisible();
	this.visible = true;
};
