//=============================================================================
// MOG_BattleResult.js
//=============================================================================

/*:
 * @plugindesc (v1.1) Modifica a tela de vítoria.
 * @author Moghunter
 *
 * @param Exp X-Axis
 * @desc Posição X-Axis da EXP.
 * @default 760
 *
 * @param Exp Y-Axis
 * @desc Posição Y-Axis da EXP.
 * @default 160
 *
 * @param Actor X-Axis
 * @desc Posição X-Axis do personagem.
 * @default 250
 *
 * @param Actor Y-Axis
 * @desc Posição Y-Axis do personagem.
 * @default 0
 *
 * @param New Skill X-Axis
 * @desc Posição X-Axis da nova habilidade.
 * @default 570
 *
 * @param New Skill Y-Axis
 * @desc Posição Y-Axis da nova habilidade.
 * @default 520
 *
 * @help
 * =============================================================================
 * +++ MOG - Battle Result (v1.1) +++
 * By Moghunter
 * https://atelierrgss.wordpress.com/
 * =============================================================================
 * Modifica a tela de vítoria.
 * =============================================================================
 * Serão necessários os arquivos. (img/system/)
 *
 * Result_A.png
 * Result_B.png
 * Result_C.png
 * Result_D.png
 * Result_E.png
 *
 * =============================================================================
 * ACTOR PICTURE
 * =============================================================================
 * As imagens deverão ser gravadas na pasta /img/picture/
 * Nomeie as imagens da seguinte forma.
 *
 * Actor_ ID
 *
 * Exemplo
 *
 * Actor_1.png
 * Actor_2.png
 * Actor_3.png
 * ...
 *
 * =============================================================================
 * HISTÓRICO
 * =============================================================================
 * (v1.1) - Correção de não apresentar a janela de status na batalha seguinte.
 */

//=============================================================================
// ** PLUGIN PARAMETERS
//=============================================================================

　　var Imported = Imported || {};
　　Imported.MOG_BattleResult = true;
　　var Moghunter = Moghunter || {};
	Moghunter.parameters = PluginManager.parameters('MOG_BattleResult');
	Moghunter.bresult_layout_x = Number(Moghunter.parameters['Layout X-Axis'] || 430);
	Moghunter.bresult_layout_y = Number(Moghunter.parameters['Layout Y-Axis'] || 266);
	Moghunter.bresult_exp_x = Number(Moghunter.parameters['Exp X-Axis'] || 760);
    Moghunter.bresult_exp_y = Number(Moghunter.parameters['Exp Y-Axis'] || 160);
	Moghunter.bresult_actor_x = Number(Moghunter.parameters['Actor X-Axis'] || 250);
    Moghunter.bresult_actor_y = Number(Moghunter.parameters['Actor Y-Axis'] || 0);
	Moghunter.bresult_skill_x = Number(Moghunter.parameters['New Skill X-Axis'] || 500);
    Moghunter.bresult_skill_y = Number(Moghunter.parameters['New Skill Y-Axis'] || 280);


//=============================================================================
// ** Game Temp
//=============================================================================

//==============================
// ** Initialize
//==============================
var _mog_bresult_gtemp_initialize = Game_Temp.prototype.initialize;
Game_Temp.prototype.initialize = function() {
    _mog_bresult_gtemp_initialize.call(this);
    this.battleResultClear()
};

//==============================
// ** battle Result Clear
//==============================
Game_Temp.prototype.battleResultClear = function() {
	this._bResult = [false,0,0];
	this._battleEnd = false;
};

Game_Actor.prototype.displayLevelUp = function(newSkills) {
};

//=============================================================================
// ** BattleManager
//=============================================================================

//==============================
// ** setup
//==============================
var _mog_brt_bmngr_setup = BattleManager.setup;
BattleManager.setup = function(troopId, canEscape, canLose) {
	_mog_brt_bmngr_setup.call(this,troopId, canEscape, canLose);
	$gameTemp.battleResultClear();
};

//==============================
// ** process Victory
//==============================
BattleManager.processVictory = function() {
	if (Imported.MOG_BattleCry) {
	    var actor = this.randomActor();
        if (actor) {SoundManager.selectVoice(actor._v_victory)};
	};
	AudioManager.fadeOutBgm(1);
	BattleManager.playVictoryMe();
	$gameTemp._battleEnd = true;
	$gameTemp._battleResult = true;
	this.getResultData();
  $gameParty.removeBattleStates();
  $gameParty.performVictory();
	this.endBattle(0);

	// AP Consumption
	var consumption = $gameSystem._difficulty == "Safe" ? 4 : $gameTemp._battle_turnAmount + 5;
	if (consumption > $gameSystem._partyAPCur) {consumption = $gameSystem._partyAPCur};
	$gameSystem._partyAPCur -= consumption;

	$gameSystem._rc_BattleWon++;
};


//==============================
// ** get Result Data
//==============================
BattleManager.getResultData = function() {
	this.makeRewards();
  $gameTemp._bResult[0] = true;
	$gameTemp._bResult[1] = this._rewards.exp;
	$gameTemp._bResult[2] = this._rewards.pp;

	if ($gameSystem._popAllow) {
		$gameTemp._popStoreTemp[0] += $gameTemp._bResult[2];
	} else {
		$gameSystem._popStoreTown += $gameTemp._bResult[2];
	}
};

BattleManager.makeRewards = function() {
  this._rewards = {};

	// Combat activity adjustment
	var rate = 1;
	var d = $gameSystem._partyAPCur
	if (d <= 0) {
		rate = 0;
	}

	if ($gameTemp._fixedExp > 0) {
		rate = 1;
		var xp = $gameTemp._fixedExp;
		$gameTemp._fixedExp = null;
	} else {
		var txp = $gameTroop.expTotal();
		var variance = (Math.random() * (1.1 - 0.9)) + 0.9;
		var xp = Math.ceil(txp * variance * rate);
	}
	this._rewards.exp = xp;

	// PP Drop
  var bpp = $gameTroop.goldTotal();

	var lv1 = 0;
	for (var i = 0; i < $gameParty.battleMembers().length; i++) {
		var battler = $gameParty.battleMembers()[i];
		if (battler) lv1 = Math.max(lv1, battler.level);
	}

	var lv2 = 0;
	for (var i = 0; i < $gameTroop.members().length; ++i) {
		var battler = $gameTroop.members()[i];
		if (battler) lv2 = Math.max(lv2, battler.level);
	}

	var dif = 1 + ((lv2 - lv1) * 0.1);
	bpp *= dif;

	var variance = (Math.random() * (1.2 - 0.8)) + 0.8;
	var pp = Math.ceil(bpp * variance * rate);
  this._rewards.pp = pp;

};

//==============================
// ** Update
//==============================
var _mog_bresult_update = BattleManager.update;
BattleManager.update = function() {
	if ($gameTemp._bResult[0]) {return};
	_mog_bresult_update.call(this);
};

//=============================================================================
// ** Scene Battle
//=============================================================================

//==============================
// ** Terminate
//==============================
var _mog_bresult_terminate = Scene_Battle.prototype.terminate;
Scene_Battle.prototype.terminate = function() {
	_mog_bresult_terminate.call(this);
	$gameTemp._bResult =  [false,0,0];
};

//==============================
// ** Update
//==============================
var _mog_bresult_sbat_update = Scene_Battle.prototype.update;
Scene_Battle.prototype.update = function() {
	_mog_bresult_sbat_update.call(this);
	if ($gameTemp._bResult[0]) {this.updateBResult()};
};

//==============================
// ** updateWindow Position
//==============================
var _mog_bresult_updateWindowPositions = Scene_Battle.prototype.updateWindowPositions;
Scene_Battle.prototype.updateWindowPositions = function() {
	_mog_bresult_updateWindowPositions.call(this);
	if ($gameTemp._battleEnd) {
		this._statusWindow.opacity -= 10;
	    this._statusWindow.contentsOpacity -= 10;
	};
};

//==============================
// ** Update BResult
//==============================
Scene_Battle.prototype.updateBResult = function() {
	if (!this._bresult) {this.createBResult()};
};

//==============================
// ** createBresult
//==============================
Scene_Battle.prototype.createBResult = function() {
	this._bresult = new BattleResult();
	this.addChild(this._bresult);
};

//=============================================================================
// * Battle Result
//=============================================================================
function BattleResult() {
    this.initialize.apply(this, arguments);
};

BattleResult.prototype = Object.create(Sprite.prototype);
BattleResult.prototype.constructor = BattleResult;

//==============================
// * Initialize
//==============================
BattleResult.prototype.initialize = function() {
    Sprite.prototype.initialize.call(this);
	if (BattleManager._spriteset && BattleManager._spriteset.combo_sprite_data) {
		BattleManager._spriteset.combo_sprite_data[0] = 1;
	};
	this._phase = 0;
	this._actorIndex = 0;
	this._skillIndex = 0;
	this.z = 50;
	this._actor = null;
	this._playME = 40;
	this.phaseAniClear();
	this._phaseAnime[0] = 1;
	this._phaseAnime[1] = 60;
	this._numbeData = [-1,-1];
	this.loadImages();
	this.createObjects();
};

//==============================
// * Phase Ani Clear
//==============================
BattleResult.prototype.phaseAniClear = function() {
	this._phaseAnime = [0,0,0];
};


//==============================
// * loadImages
//==============================
BattleResult.prototype.loadImages = function() {
	this._layout1_img = ImageManager.loadBHud("Result_A");
	this._layout2_img = ImageManager.loadBHud("Result_B");
	this._number_img = ImageManager.loadBHud("Result_C");
	this._level_img = ImageManager.loadBHud("Result_D");
	this._glow_img = ImageManager.loadBHud("Result_E");
};

//==============================
// * Press Any Key
//==============================
BattleResult.prototype.pressAnyKey = function() {
  if (TouchInput.isTriggered()) {return true};
	if (TouchInput.isCancelled()) {return true};
	if (Input.isTriggered("ok")) {return true};
	if (Input.isTriggered("cancel")) {return true};
  return false;
};

//==============================
// * Create Objects
//==============================
BattleResult.prototype.createObjects = function() {
	this.createLayout1();
	this.createLayout2();
	this.createExp();
	this.createPP();
};

//==============================
// * Create Layout
//==============================
BattleResult.prototype.createLayout1 = function() {
	this._layout1 = new Sprite(this._layout1_img);
	this._layout1.opacity = 0;
	this._layout1.x = Moghunter.bresult_layout_x;
	this._layout1.y = Moghunter.bresult_layout_y;
	this.addChild(this._layout1);
};

BattleResult.prototype.createLayout2 = function() {
	this._layout2 = new Sprite(this._layout2_img);
	this._layout2.opacity = 0;
	this._layout2.x = Moghunter.bresult_layout_x + 60;
	this._layout2.y = Moghunter.bresult_layout_y + 78;
	this.addChild(this._layout2);
};

//==============================
// * Create Exp
//==============================
BattleResult.prototype.createExp = function() {
  this._expNumber = [];
	this._expOld = 0;
	this._expOrg = [Moghunter.bresult_exp_x,Moghunter.bresult_exp_y];
	for (var i = 0; i < 8; i++) {
	   this._expNumber[i] = new Sprite(this._number_img);
	   this._expNumber[i].visible = false;
	   this._expNumber[i].x = this._expOrg[0];
	   this._expNumber[i].y = this._expOrg[1];
	   this.addChild(this._expNumber[i]);
	};

};

BattleResult.prototype.createPP = function() {
  this._ppNumber = [];
	this._ppOld = 0;
	this._ppOrg = [Moghunter.bresult_exp_x,Moghunter.bresult_exp_y + 34];
	for (var i = 0; i < 8; i++) {
	   this._ppNumber[i] = new Sprite(this._number_img);
	   this._ppNumber[i].visible = false;
	   this._ppNumber[i].x = this._ppOrg[0];
	   this._ppNumber[i].y = this._ppOrg[1];
	   this.addChild(this._ppNumber[i]);
	};

};

//==============================
// * getData
//==============================
BattleResult.prototype.getData = function() {
    this._numbeData[0] = this._number_img.width / 10;
    this._numbeData[1] = this._number_img.height;
};

//==============================
// * Update Dif
//==============================
BattleResult.prototype.update_dif = function(value,real_value,speed) {
	if (value == real_value) {return value};
	var dnspeed = 1 + (Math.abs(value - real_value) / speed);
	if (value > real_value) {value -= dnspeed;
	    if (value < real_value) {value = real_value};}
    else if (value < real_value) {value  += dnspeed;
    	if (value  > real_value) {value  = real_value};
    };
	return Math.floor(value);
};

//==============================
// * Refresh Number
//==============================
BattleResult.prototype.refresh_number = function(sprites,value,img_data,x,center) {
    numbers = Math.abs(value).toString().split("");
   	for (var i = 0; i < sprites.length ; i++) {sprites[i].visible = false;
	   if (i > numbers.length) {return};
	   var n = Number(numbers[i]);
	   sprites[i].setFrame(n * img_data[0], 0, img_data[0], img_data[1]);
	   sprites[i].visible = true;
	   if (center === 0) {
     	   var nx = -(img_data[0] * i) + (img_data[0] * numbers.length);
	   } else {
		   var nx = -(img_data[0] * i) + ((img_data[0] / 2) * numbers.length);
	   };
	   sprites[i].x = x - nx;
    };
};

//==============================
// * Update
//==============================
BattleResult.prototype.update = function() {
	Sprite.prototype.update.call(this);
	if (this._numbeData[0] === -1 && this._number_img.isReady()) {this.getData(); return};
	if (this._phaseAnime[1] > 0) {this._phaseAnime[1] --;return};
	if (this._playME > 0) {this.updateME();return};
    switch (this._phase) {
		case 0:
            this.updateLayoutAnimation();
            break;
        case 1:
            this.updatePoint();
            break;
        case 2:
            this.updateLevel();
            break;
        case 3:
            this.updateEnd();
            break;
        };
};


//==============================
// * update ME
//==============================
BattleResult.prototype.updateME = function() {
	this._playME --
	if (this._playME === 0) {
		BattleManager.replayBgmAndBgs();
	};
};

//==============================
// * Update Layout Animation
//==============================
BattleResult.prototype.updateLayoutAnimation = function() {
	this._layout1.opacity += 25;
	this._layout1.y -= 4;
	this._layout2.opacity += 25;
	this._layout2.y -= 4;
	if (this._layout1.opacity >= 255) {
		this._phase = 1;
	};
};


//==============================
// * Update Exp
//==============================
BattleResult.prototype.updatePoint = function() {
	var dif_number;

	if (this._expOld === $gameTemp._bResult[1]) {
		if (this.pressAnyKey()) {
//			SoundManager.playCursor();
			this._phase = 2;
		};
	};

	if (this.pressAnyKey() &&  $gameTemp._bResult[1] > 0) {
    this._expOld = $gameTemp._bResult[1] - 1;
		this._ppOld = $gameTemp._bResult[2] - 1;
	};

	// EXP
	dif_number = this.update_dif(this._expOld,$gameTemp._bResult[1],40);
	if (this._expOld != dif_number) {this._expOld = dif_number;
	this.refresh_number(this._expNumber,this._expOld,this._numbeData,this._expOrg[0],0)};

	// PP
	dif_number = this.update_dif(this._ppOld,$gameTemp._bResult[2],40);
	if (this._ppOld != dif_number) {
		this._ppOld = dif_number;
	 	this.refresh_number(this._ppNumber,this._ppOld,this._numbeData,this._ppOrg[0],0);
	};

};

//==============================
// * Update Level
//==============================
BattleResult.prototype.updateLevel = function() {
	if (!this._actor && this._actorIndex >= $gameParty.allMembers().length) {
		SoundManager.playCursor();
		this._phase = 3;
	} else {
		if (this._actor) {
			this.updateLevelAnimation();
		} else {
			this.gainEXP();
		};
	};
};

//==============================
// * Create Level Sprites
//==============================
BattleResult.prototype.createLevelSprites = function() {
	this.disposeLevel();
	this.createGlow();
  this.createActorSprite();
  this.createLevelLayout();
	this.playVoice();
	AudioManager.playSe({
		name: "aws",
		volume: 90,
		pitch: 100
	});
};

//==============================
// * Play Voice
//==============================
BattleResult.prototype.playVoice = function() {
	if (Imported.MOG_BattleCry) {SoundManager.selectVoice(this._actor._v_levelup)};
};

//==============================
// * Create Actor Sprite
//==============================
BattleResult.prototype.createGlow = function() {
	this._glow = new Sprite(this._glow_img);
	this._glow.opacity = 0;
	this._glow.anchor.x = 0.5;
	this._glow.anchor.y = 0.5;
	this._glow.x = Moghunter.bresult_actor_x+2;
	this._glow.y = Moghunter.bresult_actor_y+38;
	this._glow_data = [];
	this._glow_data = [0, 0, 61];
	this._glow_dataW = this._glow_img.width / this._glow_data[2];
	this._glow_dataH = this._glow_img.height;
	this.addChild(this._glow);
};

//==============================
// * Create Actor Sprite
//==============================
BattleResult.prototype.createActorSprite = function() {
	var img = ImageManager.loadBHud("ResultActor_" + this._actor._actorId)
	this._spriteActor = new Sprite(img);
	this._spriteActor.opacity = 0;
	this._spriteActor.anchor.x = 0.5;
	this._spriteActor.x = Moghunter.bresult_actor_x;
	this._spriteActor.y = Moghunter.bresult_actor_y;
	this.addChild(this._spriteActor);
};

//==============================
// * Create Level Layout
//==============================
BattleResult.prototype.createLevelLayout = function() {
	this._levelLayout = new Sprite(this._level_img);
	this._levelLayout.opacity = 0;
	this._levelLayout.anchor.x = 0.5;
	this._levelLayout.anchor.y = 0.5;
	this._levelLayout.x = Moghunter.bresult_actor_x + 102;
	this._levelLayout.y = Moghunter.bresult_actor_y + 42;
	this.addChild(this._levelLayout);
};

//==============================
// * dispose Level
//==============================
BattleResult.prototype.disposeLevel = function() {
	this.removeChild(this._glow);
	this.removeChild(this._spriteActor);
	this.removeChild(this._levelLayout);
};

//==============================
// * Update Animation
//==============================
BattleResult.prototype.updateLevelAnimation = function() {
  this._spriteActor.opacity += 10;
	this._levelLayout.opacity += 10;
	this._glow.opacity += 10;
	this.updateGlow();
	if (this.pressAnyKey()) {
		this.clearParData();
	};
};

BattleResult.prototype.updateGlow = function() {
    this._glow_data[0]++;
    if (this._glow_data[0] >= 2) {this._glow_data[0] = 0; this._glow_data[1]++}
	if (this._glow_data[1] >= this._glow_data[2]) {this._glow_data[1] = 0};
	var rcw = this._glow_dataW * this._glow_data[1];
	this._glow.setFrame(rcw,0,this._glow_dataW,this._glow_dataH);
};

//==============================
// * Sprite Move To
//==============================
BattleResult.prototype.sprite_move_to = function(value,real_value,speed) {
	if (value == real_value) {return value};
	var dnspeed = 5 + (Math.abs(value - real_value) / speed);
	if (value > real_value) {value -= dnspeed;
	    if (value < real_value) {value = real_value};}
    else if (value < real_value) {value  += dnspeed;
    	if (value  > real_value) {value  = real_value};
    };
	return Math.floor(value);
};

//==============================
// * clear Par DAta
//==============================
BattleResult.prototype.clearParData = function() {
	this._actor = null;
	this.disposeLevel();
};

//==============================
// * Gain EXP
//==============================
BattleResult.prototype.gainEXP = function() {
	var actor = $gameParty.allMembers()[this._actorIndex];
	var lvOld = actor._level
  actor.gainExp($gameTemp._bResult[1]);
  this._actorIndex += 1;
	if (actor._level > lvOld) {
		this._actor = actor;
		this.createLevelSprites();
		this.clearPoint();
	} else if (this._actorIndex >= actor.length) {
    this.clearParData();
		this._phase = 3;
	};
};

BattleResult.prototype.clearPoint = function() {
	this._layout2.opacity = 0;
	for (var i = 0; i < this._expNumber.length; i++) {
		this._expNumber[i].opacity = 0;
	};
	for (var i = 0; i < this._ppNumber.length; i++) {
		this._ppNumber[i].opacity = 0;
	};
};


//==============================
// * Update Fade
//==============================
BattleResult.prototype.updateFade = function() {
	if (this._phase === 3) {
		this._layout1.opacity = 0;
		this._layout2.opacity = 0;
		for (var i = 0; i < this._expNumber.length; i++) {
			this._expNumber[i].opacity = this._layout1.opacity;
		};
		for (var i = 0; i < this._ppNumber.length; i++) {
			this._ppNumber[i].opacity = this._layout1.opacity;
		};
		$gameTemp._bResult = [false,0,0];
	};
};

//==============================
// * Update End
//==============================
BattleResult.prototype.updateEnd = function() {
	if (this._layout1.opacity <= 0) {
       $gameTemp._bResult = [false,0,0];
	} else {
	   this.updateFade();
	};
};
