//=============================================================================
// MOG_HPGauge.js
//=============================================================================

/*:
 * @plugindesc (v1.4 *) Apresenta um medidor de HP do alvo.
 * @author Moghunter
 *
 * @param Fade Duration
 * @desc Definição do tempo para ativar o fade.
 * @default 90
 *
 * @param Enemy Layout X-Axis
 * @desc Posição X-Axis do layout.
 * @default 0
 *
 * @param Enemy Layout Y-Axis
 * @desc Posição Y-Axis do layout.
 * @default 0
 *
 * @param Enemy Meter X-Axis
 * @desc Posição X-Axis do medidor.
 * @default 8
 *
 * @param Enemy Meter Y-Axis
 * @desc Posição Y-Axis do medidor.
 * @default 10
 *
 * @param Enemy Meter 2 X-Axis
 * @desc Posição X-Axis do medidor.
 * @default 8
 *
 * @param Enemy Meter 2 Y-Axis
 * @desc Posição Y-Axis do medidor.
 * @default 10
 *
 * @help
 * =============================================================================
 * +++ MOG - HP Gauge (v1.4) +++
 * By Moghunter
 * https://mogplugins.wordpress.com
 * =============================================================================
 * Apresenta um medidor de HP do alvo.
 * Serão necessários os arquivos. (img/system/)
 *
 * HPGaugeEnemy_A.png
 * HPGaugeEnemy_B.png
 * HPGaugeEnemy_C.png
 *
 * =============================================================================
 * NOTETAGS
 * =============================================================================
 * Para ocultar o HP do inimigo use a seguinte Tag na caixa de notas
 *
 * Hide HP
 *
 * -----------------------------------------------------------------------------
 * Para ajustar a posição do medidor use a Tag abaixo.
 *
 * Hp Gauge Offset: X:Y
 *
 * Exemplo
 *
 * Hp Gauge Offset: 130:40
 *
 * ============================================================================
 * - WHAT'S  NEW (version 1.4)
 * ============================================================================
 * (NEW) - Reformulação dos plugins Parameters para a versão do RPG Maker MV 1.5+
 * (NEW) - Compatibilidade com Frontal Battle Camera plugin.
 *
 */

//=============================================================================
// ** PLUGIN PARAMETERS
//=============================================================================
　　var Imported = Imported || {};
　　Imported.MOG_HPGauge = true;
　　var Moghunter = Moghunter || {};

  　Moghunter.parameters = PluginManager.parameters('MOG_HPGauge');
    Moghunter.hpgauge_a_x = Number(Moghunter.parameters['Enemy Layout X-Axis'] || 0);
    Moghunter.hpgauge_a_y = Number(Moghunter.parameters['Enemy Layout Y-Axis'] || 0);
    Moghunter.hpgauge_b_x = Number(Moghunter.parameters['Enemy Meter X-Axis'] || 8);
    Moghunter.hpgauge_b_y = Number(Moghunter.parameters['Enemy Meter Y-Axis'] || 10);
	  Moghunter.hpgauge_c_x = Number(Moghunter.parameters['Enemy Meter 2 X-Axis'] || 8);
    Moghunter.hpgauge_c_y = Number(Moghunter.parameters['Enemy Meter 2 Y-Axis'] || 10);
	  Moghunter.hpgauge_duration = Number(Moghunter.parameters['Fade Duration'] || 90);

//=============================================================================
// ** Game_Temp
//=============================================================================

//==============================
// * Initialize
//==============================
var _mog_hpgauge_temp_initialize = Game_Temp.prototype.initialize;
Game_Temp.prototype.initialize = function() {
	_mog_hpgauge_temp_initialize.call(this);
    this._hpGauge = [true,false];
};

//=============================================================================
// ** Game_System
//=============================================================================

//==============================
// * Initialize
//==============================
var _mog_hpgauge_sys_initialize = Game_System.prototype.initialize;
Game_System.prototype.initialize = function() {
	_mog_hpgauge_sys_initialize.call(this);
    this._hpGaugeEDef = [];
};

//=============================================================================
// ** BattleManager
//=============================================================================

//==============================
// * processVictory
//==============================
var _mog_hpGauge_processVictory = BattleManager.processVictory;
BattleManager.processVictory = function() {
	 $gameTemp._hpGauge[0] = false;
	 _mog_hpGauge_processVictory.call(this);
};

//==============================
// * processAbort
//==============================
var _mog_hpGauge_processAbort = BattleManager.processAbort;
BattleManager.processAbort = function() {
	 $gameTemp._hpGauge[0] = false;
	 _mog_hpGauge_processAbort.call(this);
};

//==============================
// * processDefeat
//==============================
var _mog_hpGauge_processDefeat = BattleManager.processDefeat;
BattleManager.processDefeat = function() {
	 $gameTemp._hpGauge[0] = false;
	 _mog_hpGauge_processDefeat.call(this);
};

//=============================================================================
// ** Game Battler
//=============================================================================

//==============================
// * init Members
//==============================
var _mog_hpgauge_gbat_initMembers = Game_Battler.prototype.initMembers;
Game_Battler.prototype.initMembers = function() {
    _mog_hpgauge_gbat_initMembers.call(this);
    this._ehpgauge = [false,0,0,false,false,false];
};

//==============================
// * check HP Gauge Notes
//==============================
Game_Battler.prototype.checkHPGaugeNotes = function() {
	this.notetags().forEach(function(note) {
		if (note == "Hide HP") {this._ehpgauge[0] = false};
    if (note == "Blue HP") {this._ehpgauge[5] = true};
         var note_data = note.split(': ')
		 if (note_data[0].toLowerCase() == "hp gauge offset"){
			 var par = note_data[1].split(':');
		     this._ehpgauge[1] = Number(par[0]);
			 this._ehpgauge[2] = Number(par[1]);
		 };
	},this);
};

//==============================
// * Notetags
//==============================
Game_Battler.prototype.notetags = function() {
	if (this.isEnemy()) {return this.enemy().note.split(/[\r\n]+/)};
};

//=============================================================================
// ** Game_Enemy
//=============================================================================

//==============================
// * Setup
//==============================
var _mog_hpgauge_genemy_setup = Game_Enemy.prototype.setup;
Game_Enemy.prototype.setup = function(enemyId, x, y) {
	_mog_hpgauge_genemy_setup.call(this,enemyId, x, y);
	this._ehpgauge[0] = true;
	this.checkHPGaugeNotes();
};

//==============================
// * Die
//==============================
var _mog_hpgauge_genemy_die = Game_Enemy.prototype.die;
Game_Enemy.prototype.die = function() {
	_mog_hpgauge_genemy_die.call(this);
	$gameSystem._hpGaugeEDef[this._enemyId] = true;
};

//=============================================================================
// ** Game Action
//=============================================================================

//==============================
// * Apply
//==============================
var _mog_hpgauge_gaction_apply = Game_Action.prototype.apply;
Game_Action.prototype.apply = function(target) {
	 _mog_hpgauge_gaction_apply.call(this,target);
     if (target) {
		 if (this.isHpRecover()) {
			 target._ehpgauge[4] = true;
		 } else if (this.item() && this.item().damage.type === 5) {
			 target._ehpgauge[4] = true;
			 var subj = this.subject();
			 if (subj) {subj._ehpgauge[4] = true};
		 };
	  };
};

//=============================================================================
// ** Spriteset_Battle
//=============================================================================

//==============================
// * Create Lower Layer
//==============================
var _mog_hpGauge_createLowerLayer = Spriteset_Battle.prototype.createLowerLayer;
Spriteset_Battle.prototype.createLowerLayer = function() {
	_mog_hpGauge_createLowerLayer.call(this);
	this.createStatField();
	this.createStatSprites();
};

//==============================
// * Create EHP Field
//==============================
Spriteset_Battle.prototype.createStatField = function() {
  this._statField = new Sprite();
	this._statField.z = 100;
	this.addChild(this._statField);
};

//==============================
// * Create HP Sprites
//==============================
Spriteset_Battle.prototype.createStatSprites = function() {
	$gameTemp._hpGauge[0] = true;
	this._HPSprites = [];
  this._BSprites = [];
	this._states = [];
	this.createHPSprites();
  this.createBlueSprites();
	this.createState();
};

//==============================
// * create H PSprites Enemy
//==============================


Spriteset_Battle.prototype.createHPSprites = function() {
	for (var i = 0; i < this._enemySprites.length; i++) {
		 this._HPSprites[i] = new HPGaugeSprite(this._enemySprites[i],1);
		 this._statField.addChild(this._HPSprites[i]);
	};
};

//==============================
// * create Blue Sprites
//==============================
Spriteset_Battle.prototype.createBlueSprites = function() {
	for (var i = 0; i < this._enemySprites.length; i++) {
		 this._BSprites[i] = new BGaugeSprite(this._enemySprites[i],1);
		 this._statField.addChild(this._BSprites[i]);
	};
};

//==============================
// * create State
//==============================
Spriteset_Battle.prototype.createState = function() {
	for (var i = 0; i < this._enemySprites.length; i++) {
		 this._states[i] = new StateSprite(this._enemySprites[i]);
		 this._statField.addChild(this._states[i]);
	};
};

//==============================
// * remove HP Sprites
//==============================
Spriteset_Battle.prototype.removeHPSprites = function() {
	if (!this._HPSprites) {return};
	for (var i = 0; i < this._HPSprites.length; i++) {
		 this._statField.removeChild(this._HPSprites[i]);
		 this._statField.removeChild(this._states[i]);
	};
};

//=============================================================================
// ** HP Gauge Sprite
//=============================================================================
function HPGaugeSprite() {
    this.initialize.apply(this, arguments);
};

HPGaugeSprite.prototype = Object.create(Sprite.prototype);
HPGaugeSprite.prototype.constructor = HPGaugeSprite;

//==============================
// * Initialize
//==============================
HPGaugeSprite.prototype.initialize = function(sprite,type) {
    Sprite.prototype.initialize.call(this);
    this._sprite = sprite;
	this._slideX = 0;
	this._duration = 0;
	this.opacity = 0;
	this.type = type;
	this.z = 101;
	this.loadBitmaps();
};

//==============================
// * Battler
//==============================
HPGaugeSprite.prototype.battler = function() {
    return this._sprite._battler;
};

//==============================
// * Load Bitmaps
//==============================
HPGaugeSprite.prototype.loadBitmaps = function() {
    this._layImg = ImageManager.loadBHud("HPGaugeEnemy_A");
	this._meterImg = ImageManager.loadBHud("HPGaugeEnemy_B");
};

//==============================
// * createSprites
//==============================
HPGaugeSprite.prototype.createSprites = function() {
	this.createLayout();
	this.createMeter();
  this.createName();
};

//==============================
// * Create Layout
//==============================
HPGaugeSprite.prototype.createLayout = function() {
  this._layout = new Sprite(this._layImg);
	this._layout.x = Moghunter.hpgauge_a_x - this._layImg.width / 2;
	this._layout.y = Moghunter.hpgauge_a_y - this._layImg.height / 2;
	this.addChild(this._layout);
  this._layout.setFrame(0,0,146,16);
};

HPGaugeSprite.prototype.createName = function() {
		 this._name = new Sprite(new Bitmap(200, 40));
     this._name.x = Moghunter.hpgauge_a_x - 75;
     this._name.y = Moghunter.hpgauge_a_y - 38;
     this._name.bitmap.fontSize = 16;
     this._name.bitmap.fontFace = "FredokaSemi";
     this._name.bitmap.textColor = "#ffffff";
     this._name.bitmap.outlineWidth = 5;
     this._name.bitmap.outlineColor = "#5d4e6a";
     this._name.bitmap.drawText(this.battler().originalName(),10,0,200,40,'left');
		 this.addChild(this._name);
};

//==============================
// * Create Gauge
//==============================
HPGaugeSprite.prototype.createMeter = function() {
	this._meter = [];
	for (var i = 0; i < 2; i++) {
		this._meter[i] = new Sprite(this._meterImg);
		this._meter[i].value1 = this.battler().hp;
		this._meter[i].value2 = this.battler().mhp;
		this._meter[i].ds = 0;
		this._meter[i].w = this._meterImg.width;
		this._meter[i].h = this._meterImg.height / 2;
		this._meter[i].x = Moghunter.hpgauge_a_x + Moghunter.hpgauge_b_x - this._meterImg.width / 2;
		this._meter[i].y = Moghunter.hpgauge_a_y + Moghunter.hpgauge_b_y - this._meterImg.height / 2;
		this.addChild(this._meter[i]);
	};
    this.refreshMeter(this._meter[0],this._meter[0].value1,this.battler().mhp,1);
	this.refreshMeter(this._meter[1],this.battler().hp,this.battler().mhp,0);

	this.updateMeter();
};

//==============================
// * Need Refresh Blue Meter
//==============================
HPGaugeSprite.prototype.needRefreshBlueMeter = function() {
	if (this.battler()._ehpgauge[4]) {return true};
    if (this._meter[1].value1 != this.battler().hp) {return true};
	if (this._meter[1].value2 != this.battler().mhp) {return true};
	return false
};

//==============================
// * refresh Blue Meter
//==============================
HPGaugeSprite.prototype.refreshBlueMeter = function() {
	 this.battler()._ehpgauge[4] = false;
     this._meter[1].value1 = this.battler().hp;
	 this._meter[1].value2 = this.battler().mhp;
	 if (!$gameTemp._hpGauge[0]) {return};
	 this.refreshMeter(this._meter[0],this._meter[0].value1,this.battler().mhp,1);
	 this.refreshMeter(this._meter[1],this.battler().hp,this.battler().mhp,0);
	 this.prepareSlide();
};

//==============================
// * prepare Slide
//==============================
HPGaugeSprite.prototype.prepareSlide = function() {
	if (this.opacity < 100 || this._slideX > 0) {
	    this._slideX = -1;
	    this.opacity = 0;
	};
	this._duration = Math.min(Math.max(Moghunter.hpgauge_duration,1),999);
};

//==============================
// * Update Dif
//==============================
HPGaugeSprite.prototype.update_dif = function(value,real_value,speed,type) {
	if (value == real_value) {return value};
	var mr = type === 0 ? 1 : 0.1;
    var dnspeed = mr + (Math.abs(value - real_value) / speed);
	if (value > real_value) {value -= dnspeed;
	    if (value < real_value) {value = real_value};
	} else if (value < real_value) {value  += dnspeed;
    	if (value  > real_value) {value  = real_value};
    };
	if (type === 0) {
	    return Math.floor(value);
	} else {
		return value;
	};
};

//==============================
// * Need Refresh Red Meter
//==============================
HPGaugeSprite.prototype.needRefreshRedMeter = function() {
     if (this._meter[0].value1 != this.battler().hp) {return true};
	 return false;
};

//==============================
// * refresh Red Meter
//==============================
HPGaugeSprite.prototype.refreshRedMeter = function() {
  	 this._meter[0].value1 = this.update_dif(this._meter[0].value1,this.battler().hp, 60,1);
	 this.refreshMeter(this._meter[0],this._meter[0].value1,this.battler().mhp,1);
};

//==============================
// * Update Meter
//==============================
HPGaugeSprite.prototype.updateMeter = function() {
	 if (this.needRefreshRedMeter()) {this.refreshRedMeter()};
	 if (this.needRefreshBlueMeter()) {this.refreshBlueMeter()};
};

//==============================
// * Refresh Meter
//==============================
HPGaugeSprite.prototype.refreshMeter = function(sprite,value1,value2,h) {
     var wd = value1 * sprite.w / value2
	 sprite.setFrame(0,sprite.h * h,wd,sprite.h);
};

//==============================
// * Is Visible
//==============================
HPGaugeSprite.prototype.isVisible = function() {
	if (!this.battler()) {return false};
	if (this.battler().isHidden()) {return false};
    if (!this.battler()._ehpgauge[0]) {return false};
	return true;
};

//==============================
// * force Visible
//==============================
HPGaugeSprite.prototype.forceVisible = function() {
     if (this.battler()._ehpgauge[3]) {return true};
	 if (this.battler().isSelected() && !this.battler().isDead()) {return true};
	 return false;
};

//==============================
// * pos X
//==============================
HPGaugeSprite.prototype.posX = function() {
    return this._sprite.x + this.battler()._ehpgauge[1];
};

//==============================
// * pos Y
//==============================
HPGaugeSprite.prototype.posY = function() {
    return this._sprite.y + this.battler()._ehpgauge[2];
};

//==============================
// * Update Slide
//==============================
HPGaugeSprite.prototype.updateSlide = function() {
	this._slideX += 2;
	this.opacity += 10;
	if (this._slideX >= 0) {
		this._slideX = 0;
		this.opacity = 255;
	};
};

//==============================
// * Update Fade
//==============================
HPGaugeSprite.prototype.updateFade = function() {
	this.opacity -= 10;
};

//==============================
// * Update Slide
//==============================
HPGaugeSprite.prototype.updateAnimation = function() {
    if (this._slideX < 0) {this.updateSlide();return};
	if (this._duration > 0) {this._duration--;return};
	if (this.opacity > 0) {this.updateFade()};
};

//==============================
// * Update Force Visible
//==============================
HPGaugeSprite.prototype.updateForceVisible = function() {
    this.opacity = 255;
    this._duration = 0;
    this._slideX = 0;
};

//==============================
// * Update Position
//==============================
HPGaugeSprite.prototype.updatePosition = function() {
   if (this.forceVisible()) {
       this.updateForceVisible();
   } else {
	   this.updateAnimation();
   };
   this.x = this.posX();
   this.y = this.posY();
};

//==============================
// * Update
//==============================
HPGaugeSprite.prototype.update = function() {
    Sprite.prototype.update.call(this);
	this.visible = this.isVisible();
	if (!this.battler()) {return};
	if (!this._layout && this._meterImg.isReady()) {this.createSprites()};
	if (this._meter) {this.updateMeter()};
    this.updatePosition();
};

//=============================================================================
// ** Blue Gauge Sprite
//=============================================================================
function BGaugeSprite() {
    this.initialize.apply(this, arguments);
};

BGaugeSprite.prototype = Object.create(Sprite.prototype);
BGaugeSprite.prototype.constructor = BGaugeSprite;

//==============================
// * Initialize
//==============================
BGaugeSprite.prototype.initialize = function(sprite,type) {
    Sprite.prototype.initialize.call(this);
  this._sprite = sprite;
	this._slideX = 0;
	this._duration = 0;
	this.opacity = 0;
	this.type = type;
	this.z = 101;
	this.loadBitmaps();
};

//==============================
// * Battler
//==============================
BGaugeSprite.prototype.battler = function() {
    return this._sprite._battler;
};

//==============================
// * Load Bitmaps
//==============================
BGaugeSprite.prototype.loadBitmaps = function() {
  this._layImg = ImageManager.loadBHud("HPGaugeEnemy_A");
	this._meterImg = ImageManager.loadBHud("HPGaugeEnemy_C");
};

//==============================
// * createSprites
//==============================
BGaugeSprite.prototype.createSprites = function() {
	this.createLayout();
	this.createMeter();
};

//==============================
// * Create Layout
//==============================
BGaugeSprite.prototype.createLayout = function() {
  this._layout = new Sprite(this._layImg);
	this._layout.x = Moghunter.hpgauge_a_x - 42;
	this._layout.y = Moghunter.hpgauge_a_y - 5;
	this.addChild(this._layout);
  this._layout.setFrame(0,16,146,8);
};

//==============================
// * Create Gauge
//==============================
BGaugeSprite.prototype.createMeter = function() {
	this._meter = new Sprite(this._meterImg);
	this._meter.w = this._meterImg.width;
  this._meter.h = this._meterImg.height;
	this._meter.x = this._layout.x + 19;
	this._meter.y = this._layout.y + 2;
	this.addChild(this._meter);
};

//==============================
// * Update Dif
//==============================
BGaugeSprite.prototype.update_dif = function(value,real_value,speed,type) {
	if (value == real_value) {return value};
	var mr = type === 0 ? 1 : 0.1;
    var dnspeed = mr + (Math.abs(value - real_value) / speed);
	if (value > real_value) {value -= dnspeed;
	    if (value < real_value) {value = real_value};
	} else if (value < real_value) {value  += dnspeed;
    	if (value  > real_value) {value  = real_value};
    };
	if (type === 0) {
	    return Math.floor(value);
	} else {
		return value;
	};
};

//==============================
// * Refresh Meter
//==============================
BGaugeSprite.prototype.updateMeter = function() {
  this._meter.value1 = this.battler()._blue;
  this._meter.value2 = this.battler()._bluemax;
  this.refreshMeter(this._meter, this._meter.value1, this._meter.value2);
};

BGaugeSprite.prototype.refreshMeter = function(sprite,value1,value2) {
  var wd = value1 * sprite.w / value2
	sprite.setFrame(0,0,wd,sprite.h);
};

//==============================
// * Is Visible
//==============================
BGaugeSprite.prototype.isVisible = function() {
	if (!this.battler()) {return false};
	if (this.battler().isHidden()) {return false};
  if (!this.battler()._ehpgauge[5]) {return false};
	return true;
};

//==============================
// * force Visible
//==============================
BGaugeSprite.prototype.forceVisible = function() {
  if (this.battler()._ehpgauge[3]) {return true};
	if (this.battler().isSelected() && !this.battler().isDead()) {return true};
	return false;
};

//==============================
// * pos X
//==============================
BGaugeSprite.prototype.posX = function() {
    return this._sprite.x + this.battler()._ehpgauge[1];
};

//==============================
// * pos Y
//==============================
BGaugeSprite.prototype.posY = function() {
    return this._sprite.y + this.battler()._ehpgauge[2];
};

//==============================
// * Update Slide
//==============================
BGaugeSprite.prototype.updateSlide = function() {
	this._slideX += 2;
	this.opacity += 10;
	if (this._slideX >= 0) {
		this._slideX = 0;
		this.opacity = 255;
	};
};

//==============================
// * Update Fade
//==============================
BGaugeSprite.prototype.updateFade = function() {
	this.opacity -= 10;
};

//==============================
// * Update Slide
//==============================
BGaugeSprite.prototype.updateAnimation = function() {
    if (this._slideX < 0) {this.updateSlide();return};
	if (this._duration > 0) {this._duration--;return};
	if (this.opacity > 0) {this.updateFade()};
};

//==============================
// * Update Force Visible
//==============================
BGaugeSprite.prototype.updateForceVisible = function() {
    this.opacity = 255;
    this._duration = 0;
    this._slideX = 0;
};

//==============================
// * Update Position
//==============================
BGaugeSprite.prototype.updatePosition = function() {
   if (this.forceVisible()) {
       this.updateForceVisible();
   } else {
	   this.updateAnimation();
   };
   this.x = this.posX();
   this.y = this.posY();
};

//==============================
// * Update
//==============================
BGaugeSprite.prototype.update = function() {
    Sprite.prototype.update.call(this);
	this.visible = this.isVisible();
	if (!this.battler()) {return};
	if (!this._layout && this._meterImg.isReady()) {this.createSprites()};
	if (this._meter) {this.updateMeter()};
  this.updatePosition();
};

//=============================================================================
// ** State Sprite
//=============================================================================
function StateSprite() {
    this.initialize.apply(this, arguments);
};

StateSprite.prototype = Object.create(Sprite.prototype);
StateSprite.prototype.constructor = StateSprite;

//==============================
// * Initialize
//==============================
StateSprite.prototype.initialize = function(sprite) {
    Sprite.prototype.initialize.call(this);
  this._sprite = sprite;
	this._duration = 0;
	this.opacity = 0;
	this.z = 100;
	this.loadBitmaps();
};

//==============================
// * Battler
//==============================
StateSprite.prototype.battler = function() {
    return this._sprite._battler;
};

//==============================
// * Load Bitmaps
//==============================
StateSprite.prototype.loadBitmaps = function() {
    this._state_img = ImageManager.loadSystem("IconSet");
};

//==============================
// * Create Layout
//==============================
StateSprite.prototype.createSprites = function() {
    this._stateIcons = [];
    this._stateturn = [];
    this._stateE = false;
};

//==============================
// * Refresh State
//==============================
StateSprite.prototype.refreshStates = function() {
	if (!this.needRefresh()) {return};
  for (i = 0; i < this._stateIcons.length; i++){
		this.removeChild(this._stateIcons[i]);
		this.removeChild(this._stateturn[i]);
	};
	var w = Window_Base._iconWidth;
	var icons = this.battler().allIcons().slice(0,w);
	var n = this.battler().states();
	m = Math.min(icons.length,3);

	for (i = 0; i < m; i++){
		 var state = n[i];
		 this._stateIcons[i] = new Sprite(this._state_img);
	     var sx = icons[i] % 16 * w;
		 var sy = Math.floor(icons[i] / 16) * w;
		 this._stateIcons[i].setFrame(sx, sy, w, w);
		 this._stateIcons[i].x = (w - 6) * i;
		 this.addChild(this._stateIcons[i]);
		 var tx = this._stateIcons[i].x + 23;
		 var ty = this._stateIcons[i].y + 8;
		 this._stateturn[i] = new Sprite(new Bitmap(260,32));
		 this._stateturn[i].bitmap.fontSize = 14;
		 this._stateturn[i].bitmap.textColor = '#ffffff';
		 this._stateturn[i].bitmap.outlineColor = 'rgba(0,0,0,0.5)';
		 this._stateturn[i].bitmap.outlineWidth = 4;
		 var temp = this.battler().stateTurns(state.id);
		 this._stateturn[i].bitmap.drawText(temp,tx,ty,260,32,"left");
		 this.addChild(this._stateturn[i]);
	};
	this._stateE = true;
};

//==============================
// * Is Visible
//==============================
StateSprite.prototype.isVisible = function() {
	if (!this.battler()) {return false};
	if (this.battler().isHidden()) {return false};
    if (!this.battler()._ehpgauge[0]) {return false};
	return true;
};

//==============================
// * force Visible
//==============================
StateSprite.prototype.forceVisible = function() {
     if (this.battler()._ehpgauge[3]) {return true};
	 if (this.battler().isSelected() && !this.battler().isDead()) {return true};
	 return false;
};

//==============================
// * check Refresh
//==============================
StateSprite.prototype.needRefresh = function() {
	if (this.battler().isSelected() && !this.battler().isDead() && !this._stateE) {return true};
	return false;
};

//==============================
// * Update Fade
//==============================
StateSprite.prototype.updateFade = function() {
	this.opacity -= 10;
};

//==============================
// * Update Slide
//==============================
StateSprite.prototype.updateAnimation = function() {
	if (this._duration > 0) {this._duration--;return};
	if (this.opacity > 0) {this.updateFade()};
};

//==============================
// * Update Force Visible
//==============================
StateSprite.prototype.updateForceVisible = function() {
    this.opacity = 255;
    this._duration = 0;
};

//==============================
// * Update Position
//==============================
StateSprite.prototype.updatePosition = function() {
   if (this.forceVisible()) {
       this.updateForceVisible();
	   this.refreshStates();
   } else {
	   this.updateAnimation();
	   this._stateE = false;
   };
   this.x = this._sprite.x + this.battler()._ehpgauge[1] - 71;
   this.y = this._sprite.y + this.battler()._ehpgauge[2] - 130;
};

//==============================
// * Update
//==============================
StateSprite.prototype.update = function() {
    Sprite.prototype.update.call(this);
	this.visible = this.isVisible();
	if (!this.battler()) {return};
	if (!this._stateIcons) {this.createSprites()};
    this.updatePosition();
};
